<?php
/* Driver template for the PHP_Haanga_rGenerator parser generator. (PHP port of LEMON)
*/

/**
 * This can be used to store both the string representation of
 * a token, and any useful meta-data associated with the token.
 *
 * meta-data should be stored as an array
 */
class Haanga_yyToken implements ArrayAccess
{
    public $string = '';
    public $metadata = array();

    function __construct($s, $m = array())
    {
        if ($s instanceof Haanga_yyToken) {
            $this->string = $s->string;
            $this->metadata = $s->metadata;
        } else {
            $this->string = (string) $s;
            if ($m instanceof Haanga_yyToken) {
                $this->metadata = $m->metadata;
            } elseif (is_array($m)) {
                $this->metadata = $m;
            }
        }
    }

    function __toString()
    {
        return $this->string;
    }

    function offsetExists($offset)
    {
        return isset($this->metadata[$offset]);
    }

    function offsetGet($offset)
    {
        return $this->metadata[$offset];
    }

    function offsetSet($offset, $value)
    {
        if ($offset === null) {
            if (isset($value[0])) {
                $x = ($value instanceof Haanga_yyToken) ?
                    $value->metadata : $value;
                $this->metadata = array_merge($this->metadata, $x);
                return;
            }
            $offset = count($this->metadata);
        }
        if ($value === null) {
            return;
        }
        if ($value instanceof Haanga_yyToken) {
            if ($value->metadata) {
                $this->metadata[$offset] = $value->metadata;
            }
        } elseif ($value) {
            $this->metadata[$offset] = $value;
        }
    }

    function offsetUnset($offset)
    {
        unset($this->metadata[$offset]);
    }
}

/** The following structure represents a single element of the
 * parser's stack.  Information stored includes:
 *
 *   +  The state number for the parser at this level of the stack.
 *
 *   +  The value of the token stored at this level of the stack.
 *      (In other words, the "major" token.)
 *
 *   +  The semantic value stored at this level of the stack.  This is
 *      the information used by the action routines in the grammar.
 *      It is sometimes called the "minor" token.
 */
class Haanga_yyStackEntry
{
    public $stateno;       /* The state-number */
    public $major;         /* The major token value.  This is the code
                     ** number for the token at this stack level */
    public $minor; /* The user-supplied minor token value.  This
                     ** is the value of the token  */
};

// code external to the class is included here
#line 2 "Parser.y"

/*
  +---------------------------------------------------------------------------------+
  | Copyright (c) 2010 César Rodas and Menéame Comunicacions S.L.                   |
  +---------------------------------------------------------------------------------+
  | Redistribution and use in source and binary forms, with or without              |
  | modification, are permitted provided that the following conditions are met:     |
  | 1. Redistributions of source code must retain the above copyright               |
  |    notice, this list of conditions and the following disclaimer.                |
  |                                                                                 |
  | 2. Redistributions in binary form must reproduce the above copyright            |
  |    notice, this list of conditions and the following disclaimer in the          |
  |    documentation and/or other materials provided with the distribution.         |
  |                                                                                 |
  | 3. All advertising materials mentioning features or use of this software        |
  |    must display the following acknowledgement:                                  |
  |    This product includes software developed by César D. Rodas.                  |
  |                                                                                 |
  | 4. Neither the name of the César D. Rodas nor the                               |
  |    names of its contributors may be used to endorse or promote products         |
  |    derived from this software without specific prior written permission.        |
  |                                                                                 |
  | THIS SOFTWARE IS PROVIDED BY CÉSAR D. RODAS ''AS IS'' AND ANY                   |
  | EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED       |
  | WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE          |
  | DISCLAIMED. IN NO EVENT SHALL CÉSAR D. RODAS BE LIABLE FOR ANY                  |
  | DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES      |
  | (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    |
  | LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND     |
  | ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT      |
  | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS   |
  | SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE                     |
  +---------------------------------------------------------------------------------+
  | Authors: César Rodas <crodas@php.net>                                           |
  +---------------------------------------------------------------------------------+
*/
#line 136 "Parser.php"

// declare_class is output here
#line 39 "Parser.y"
 class Haanga_Compiler_Parser #line 141 "Parser.php"
{
/* First off, code is included which follows the "include_class" declaration
** in the input file. */
#line 40 "Parser.y"

    protected $lex;
    protected $file;

    function __construct($lex, $file='')
    {
        $this->lex  = $lex;
        $this->file = $file;
    }

    function Error($text)
    {
        throw new Haanga_Compiler_Exception($text.' in '.$this->file.':'.$this->lex->getLine());
    }

#line 162 "Parser.php"

/* Next is all token values, as class constants
*/
/* 
** These constants (all generated automatically by the parser generator)
** specify the various kinds of tokens (terminals) that the parser
** understands. 
**
** Each symbol here is a terminal symbol in the grammar.
*/
    const T_TAG_OPEN                     =  1;
    const T_NOT                          =  2;
    const T_AND                          =  3;
    const T_OR                           =  4;
    const T_EQ                           =  5;
    const T_NE                           =  6;
    const T_GT                           =  7;
    const T_GE                           =  8;
    const T_LT                           =  9;
    const T_LE                           = 10;
    const T_IN                           = 11;
    const T_PLUS                         = 12;
    const T_MINUS                        = 13;
    const T_CONCAT                       = 14;
    const T_TIMES                        = 15;
    const T_DIV                          = 16;
    const T_MOD                          = 17;
    const T_PIPE                         = 18;
    const T_BITWISE                      = 19;
    const T_HTML                         = 20;
    const T_COMMENT                      = 21;
    const T_PRINT_OPEN                   = 22;
    const T_PRINT_CLOSE                  = 23;
    const T_EXTENDS                      = 24;
    const T_TAG_CLOSE                    = 25;
    const T_INCLUDE                      = 26;
    const T_AUTOESCAPE                   = 27;
    const T_CUSTOM_END                   = 28;
    const T_CUSTOM_TAG                   = 29;
    const T_AS                           = 30;
    const T_CUSTOM_BLOCK                 = 31;
    const T_SPACEFULL                    = 32;
    const T_WITH                         = 33;
    const T_SET                          = 34;
    const T_ASSIGN                       = 35;
    const T_LOAD                         = 36;
    const T_FOR                          = 37;
    const T_COMMA                        = 38;
    const T_STEP                         = 39;
    const T_EMPTY                        = 40;
    const T_IF                           = 41;
    const T_ELSE                         = 42;
    const T_IFCHANGED                    = 43;
    const T_IFEQUAL                      = 44;
    const T_IFNOTEQUAL                   = 45;
    const T_BLOCK                        = 46;
    const T_FILTER                       = 47;
    const T_REGROUP                      = 48;
    const T_BY                           = 49;
    const T_COLON                        = 50;
    const T_TRUE                         = 51;
    const T_FALSE                        = 52;
    const T_STRING                       = 53;
    const T_INTL                         = 54;
    const T_RPARENT                      = 55;
    const T_LPARENT                      = 56;
    const T_OBJ                          = 57;
    const T_DOT                          = 58;
    const T_CLASS                        = 59;
    const T_BRACKETS_OPEN                = 60;
    const T_BRACKETS_CLOSE               = 61;
    const T_ALPHA                        = 62;
    const T_DOTDOT                       = 63;
    const T_NUMERIC                      = 64;
    const YY_NO_ACTION = 356;
    const YY_ACCEPT_ACTION = 355;
    const YY_ERROR_ACTION = 354;

/* Next are that tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N < self::YYNSTATE                              Shift N.  That is,
**                                                        push the lookahead
**                                                        token onto the stack
**                                                        and goto state N.
**
**   self::YYNSTATE <= N < self::YYNSTATE+self::YYNRULE   Reduce by rule N-YYNSTATE.
**
**   N == self::YYNSTATE+self::YYNRULE                    A syntax error has occurred.
**
**   N == self::YYNSTATE+self::YYNRULE+1                  The parser accepts its
**                                                        input. (and concludes parsing)
**
**   N == self::YYNSTATE+self::YYNRULE+2                  No such action.  Denotes unused
**                                                        slots in the yy_action[] table.
**
** The action table is constructed as a single large static array $yy_action.
** Given state S and lookahead X, the action is computed as
**
**      self::$yy_action[self::$yy_shift_ofst[S] + X ]
**
** If the index value self::$yy_shift_ofst[S]+X is out of range or if the value
** self::$yy_lookahead[self::$yy_shift_ofst[S]+X] is not equal to X or if
** self::$yy_shift_ofst[S] is equal to self::YY_SHIFT_USE_DFLT, it means that
** the action is not in the table and that self::$yy_default[S] should be used instead.  
**
** The formula above is for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the static $yy_reduce_ofst array is used in place of
** the static $yy_shift_ofst array and self::YY_REDUCE_USE_DFLT is used in place of
** self::YY_SHIFT_USE_DFLT.
**
** The following are the tables generated in this section:
**
**  self::$yy_action        A single table containing all actions.
**  self::$yy_lookahead     A table containing the lookahead for each entry in
**                          yy_action.  Used to detect hash collisions.
**  self::$yy_shift_ofst    For each state, the offset into self::$yy_action for
**                          shifting terminals.
**  self::$yy_reduce_ofst   For each state, the offset into self::$yy_action for
**                          shifting non-terminals after a reduce.
**  self::$yy_default       Default action for each state.
*/
    const YY_SZ_ACTTAB = 1402;
static public $yy_action = array(
 /*     0 */    49,   57,   47,   66,  163,   39,  188,   41,  186,   61,
 /*    10 */    63,  225,   95,   67,   28,   28,  104,   33,  165,   40,
 /*    20 */    35,   37,   56,   52,   53,   49,  232,   47,   66,  157,
 /*    30 */    39,  195,   41,  186,   61,   63,  247,   95,   67,  228,
 /*    40 */   150,  210,   33,  158,   40,   35,   37,   56,   52,   53,
 /*    50 */    49,  229,   47,   66,  179,   39,  212,   41,  186,   61,
 /*    60 */    63,  235,   95,   67,   38,  355,   86,   33,  183,   40,
 /*    70 */    35,   37,   56,   52,   53,   49,   57,   47,   66,  185,
 /*    80 */    39,  200,   41,  186,   61,   63,  108,   95,   67,  116,
 /*    90 */   117,   68,   33,  181,   40,   35,   37,   56,   52,   53,
 /*   100 */    49,  230,   47,   66,  171,   39,  106,   41,  186,   61,
 /*   110 */    63,   57,   95,   67,  105,  215,  173,   33,  239,   40,
 /*   120 */    35,   37,   56,   52,   53,   49,  209,   47,   66,  146,
 /*   130 */    39,  242,   41,  186,   61,   63,  214,   95,   67,  102,
 /*   140 */   253,  251,   33,  147,   40,   35,   37,   56,   52,   53,
 /*   150 */    49,   59,   47,   66,  153,   39,  110,   41,  186,   61,
 /*   160 */    63,   57,   95,   67,  191,  199,  248,   33,  190,   40,
 /*   170 */    35,   37,   56,   52,   53,   49,   60,   47,   66,  184,
 /*   180 */    39,  244,   41,  186,   61,   63,  204,   95,   67,  246,
 /*   190 */   103,  115,   33,   25,   40,   35,   37,   56,   52,   53,
 /*   200 */    49,   50,   47,   66,  138,   39,  243,   41,  186,   61,
 /*   210 */    63,  252,   95,   67,  236,   98,  194,   33,   45,   40,
 /*   220 */    35,   37,   56,   52,   53,   49,  154,   47,   66,  139,
 /*   230 */    39,  351,   41,  186,   61,   63,   57,   95,   67,  238,
 /*   240 */    54,  237,   33,   97,   40,   35,   37,   56,   52,   53,
 /*   250 */    49,  233,   47,   66,  160,   39,  107,   41,  186,   61,
 /*   260 */    63,   45,   95,   67,  264,   99,  113,   33,  189,   40,
 /*   270 */    35,   37,   56,   52,   53,   49,   76,   47,   66,  164,
 /*   280 */    39,  224,   41,  186,   61,   63,  203,   95,   67,   89,
 /*   290 */    71,   83,   33,  241,   40,   35,   37,   56,   52,   53,
 /*   300 */    49,   82,   47,   66,  187,   39,  185,   41,  186,   61,
 /*   310 */    63,   78,   95,   67,   73,   84,   80,   33,  174,   40,
 /*   320 */    35,   37,   56,   52,   53,   49,   75,   47,   66,  167,
 /*   330 */    39,   74,   41,  186,   61,   63,   72,   95,   67,   81,
 /*   340 */    85,   77,   33,  185,   40,   35,   37,   56,   52,   53,
 /*   350 */    49,   79,   47,   66,  176,   39,   90,   41,  186,   61,
 /*   360 */    63,   92,   95,   67,   88,   91,   93,   33,  185,   40,
 /*   370 */    35,   37,   56,   52,   53,   49,   87,   47,   66,  140,
 /*   380 */    39,  185,   41,  186,   61,   63,  185,   95,   67,  185,
 /*   390 */   185,  185,   33,  185,   40,   35,   37,   56,   52,   53,
 /*   400 */    49,  185,   47,   66,  149,   39,  185,   41,  186,   61,
 /*   410 */    63,  185,   95,   67,  185,  185,  185,   33,  185,   40,
 /*   420 */    35,   37,   56,   52,   53,   49,  185,   47,   66,   94,
 /*   430 */    39,  185,   41,  186,   61,   63,  185,   95,   67,  185,
 /*   440 */   185,  185,   33,  185,   40,   35,   37,   56,   52,   53,
 /*   450 */    49,  185,   47,   66,  166,   39,  161,   41,  186,   61,
 /*   460 */    63,  185,   95,   67,  185,  185,  185,   33,  192,   40,
 /*   470 */    35,   37,   56,   52,   53,   32,   26,   29,   29,   29,
 /*   480 */    29,   29,   29,   29,   30,   30,   30,   27,   27,   27,
 /*   490 */    28,   28,   49,  185,   47,   66,  142,   39,  185,   41,
 /*   500 */   186,   61,   63,  185,   95,   67,   57,  219,  100,   33,
 /*   510 */   185,   40,   35,   37,   56,   52,   53,   70,   70,   69,
 /*   520 */    48,   49,   51,   47,   66,   58,   39,  250,   41,  186,
 /*   530 */    61,   63,  185,   95,   67,  185,  185,   62,   33,  185,
 /*   540 */    40,   35,   37,   56,   52,   53,   32,   26,   29,   29,
 /*   550 */    29,   29,   29,   29,   29,   30,   30,   30,   27,   27,
 /*   560 */    27,   28,   28,   27,   27,   27,   28,   28,  111,   49,
 /*   570 */   185,   47,   66,  177,   39,  185,   41,  186,   61,   63,
 /*   580 */   185,   95,   67,  185,  185,  185,   33,  127,   40,   35,
 /*   590 */    37,   56,   52,   53,   49,  185,   47,   66,  175,   39,
 /*   600 */   185,   41,  186,   61,   63,  185,   95,   67,  263,  126,
 /*   610 */   220,   33,  185,   40,   35,   37,   56,   52,   53,   32,
 /*   620 */    26,   29,   29,   29,   29,   29,   29,   29,   30,   30,
 /*   630 */    30,   27,   27,   27,   28,   28,   26,   29,   29,   29,
 /*   640 */    29,   29,   29,   29,   30,   30,   30,   27,   27,   27,
 /*   650 */    28,   28,   29,   29,   29,   29,   29,   29,   29,   30,
 /*   660 */    30,   30,   27,   27,   27,   28,   28,  161,  231,  231,
 /*   670 */    24,  231,  185,  185,  185,  185,  185,  185,  175,  234,
 /*   680 */   185,  161,  231,  231,   64,  231,  231,  185,  257,  126,
 /*   690 */   220,  185,   46,  185,  185,  185,  231,  231,  185,  231,
 /*   700 */   231,  155,  231,  185,  185,  222,  222,  228,  150,  185,
 /*   710 */   172,  161,  126,  220,  231,  185,  231,  185,  219,  261,
 /*   720 */   261,  228,  150,  213,   31,  131,  231,  231,   65,  231,
 /*   730 */   231,  185,  219,  185,  185,  185,  134,  185,  185,  185,
 /*   740 */   185,  161,  129,  156,  231,  201,  263,  126,  220,  222,
 /*   750 */   222,  228,  150,  112,  254,  185,  231,  231,  185,  231,
 /*   760 */   231,  185,  219,  185,  223,   42,   46,  185,  227,  185,
 /*   770 */   185,  161,  185,  221,  231,  126,  220,  185,  185,  222,
 /*   780 */   222,  228,  150,  114,  185,  185,  231,  231,  196,  231,
 /*   790 */   231,  185,  219,  185,  197,  202,   46,  201,  185,  126,
 /*   800 */   220,  161,  185,  180,  231,  126,  220,  185,  185,  222,
 /*   810 */   222,  228,  150,  109,  126,  220,  231,  231,  185,  231,
 /*   820 */   231,  226,  219,  185,  137,  208,  207,  205,  206,  211,
 /*   830 */   217,  218,  216,  161,  231,  185,  240,  185,   96,  222,
 /*   840 */   222,  228,  150,  185,  168,  101,  185,  161,  231,  231,
 /*   850 */   231,  231,  219,  185,  245,  126,  220,  231,  231,  185,
 /*   860 */   231,  161,  231,  231,  185,  231,  231,  185,  185,  185,
 /*   870 */   185,  222,  222,  228,  150,  231,  231,  231,  182,  231,
 /*   880 */   231,  143,  231,  185,  219,  222,  222,  228,  150,  126,
 /*   890 */   220,  231,  126,  220,  231,  151,  231,  162,  219,  261,
 /*   900 */   261,  228,  150,  170,  185,  185,  126,  220,  126,  220,
 /*   910 */   231,  185,  219,  185,  175,  185,  125,  185,  259,  170,
 /*   920 */   185,  185,  249,  262,  263,  126,  220,  185,  185,  185,
 /*   930 */   175,  185,  118,  170,  259,  185,  185,  185,  249,  262,
 /*   940 */   263,  126,  220,  185,  175,  185,  124,  170,  259,  185,
 /*   950 */   135,  185,  249,  262,  263,  126,  220,  185,  175,  185,
 /*   960 */   130,  175,  259,  169,  185,  170,  249,  262,  263,  126,
 /*   970 */   220,  263,  126,  220,  126,  220,  175,  185,  121,  132,
 /*   980 */   259,  170,  185,  185,  249,  262,  263,  126,  220,  185,
 /*   990 */   126,  220,  175,  185,  120,  170,  259,  185,  185,  185,
 /*  1000 */   249,  262,  263,  126,  220,  185,  175,  185,  123,  170,
 /*  1010 */   259,  185,  185,  185,  249,  262,  263,  126,  220,  185,
 /*  1020 */   175,  185,  122,  170,  259,  185,  185,  185,  249,  262,
 /*  1030 */   263,  126,  220,  185,  175,  185,  260,  170,  259,  185,
 /*  1040 */   185,  185,  249,  262,  263,  126,  220,  185,  175,  185,
 /*  1050 */   119,  170,  259,  185,  185,  185,  249,  262,  263,  126,
 /*  1060 */   220,  185,  175,  185,  185,  170,  259,  185,  185,  185,
 /*  1070 */    36,  262,  263,  126,  220,  185,  175,  185,  185,  170,
 /*  1080 */   259,  185,  185,  185,  144,  262,  263,  126,  220,  185,
 /*  1090 */   175,  185,  185,  170,  259,  185,  185,  185,   34,  262,
 /*  1100 */   263,  126,  220,  185,  175,  185,  148,  161,  259,  185,
 /*  1110 */   254,  185,  141,  262,  263,  126,  220,  126,  220,  185,
 /*  1120 */   223,   43,  231,  231,  227,  231,  185,  185,  254,  221,
 /*  1130 */   185,  126,  220,  178,  185,  185,  185,  185,  223,   44,
 /*  1140 */   231,  185,  227,  255,  126,  220,  185,  221,  185,  126,
 /*  1150 */   220,  185,  185,  223,  185,  185,  231,  227,  219,  159,
 /*  1160 */   185,  185,  221,  185,  126,  220,  185,  185,  185,  223,
 /*  1170 */   185,  185,  152,  227,  185,  185,  185,  185,  221,  185,
 /*  1180 */   126,  220,  223,  185,  185,  256,  227,  185,  185,  185,
 /*  1190 */   185,  221,  185,  126,  220,  223,  185,   20,  185,  227,
 /*  1200 */   136,  185,  185,  185,  221,  185,  126,  220,  185,  185,
 /*  1210 */   223,  185,  185,  185,  227,  258,  198,  193,   55,  221,
 /*  1220 */   185,  126,  220,  185,  128,  223,  185,  185,  133,  227,
 /*  1230 */   185,  196,  185,   12,  221,  175,  126,  220,  145,  175,
 /*  1240 */   201,   11,  126,  220,   19,  263,  126,  220,   16,  263,
 /*  1250 */   126,  220,  198,  193,   55,    4,  185,  185,  185,  185,
 /*  1260 */   198,  193,   55,  198,  193,   55,    7,  198,  193,   55,
 /*  1270 */    14,  185,  185,  185,  198,  193,   55,   21,  185,  185,
 /*  1280 */   185,  185,  185,  185,  185,  198,  193,   55,   17,  198,
 /*  1290 */   193,   55,    3,  185,  185,  185,  198,  193,   55,   23,
 /*  1300 */   185,  185,  185,  185,  185,  185,  185,  198,  193,   55,
 /*  1310 */    13,  198,  193,   55,   22,  185,  185,  185,  198,  193,
 /*  1320 */    55,    9,  185,  185,  185,  185,  185,  185,  185,  198,
 /*  1330 */   193,   55,    2,  198,  193,   55,   10,  185,  185,  185,
 /*  1340 */   198,  193,   55,   15,  185,  185,  185,  185,  185,  185,
 /*  1350 */   185,  198,  193,   55,   18,  198,  193,   55,    8,  185,
 /*  1360 */   185,  185,  198,  193,   55,    5,  185,  185,  185,  185,
 /*  1370 */   185,  185,  185,  198,  193,   55,    1,  198,  193,   55,
 /*  1380 */     6,  185,  185,  185,  198,  193,   55,  185,  185,  185,
 /*  1390 */   185,  185,  185,  185,  185,  198,  193,   55,  185,  198,
 /*  1400 */   193,   55,
    );
    static public $yy_lookahead = array(
 /*     0 */    24,   18,   26,   27,   28,   29,   23,   31,   32,   33,
 /*    10 */    34,   68,   36,   37,   18,   19,   25,   41,   42,   43,
 /*    20 */    44,   45,   46,   47,   48,   24,   93,   26,   27,   28,
 /*    30 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   53,
 /*    40 */    54,   25,   41,   42,   43,   44,   45,   46,   47,   48,
 /*    50 */    24,   93,   26,   27,   28,   29,   25,   31,   32,   33,
 /*    60 */    34,   25,   36,   37,   11,   66,   67,   41,   42,   43,
 /*    70 */    44,   45,   46,   47,   48,   24,   18,   26,   27,   28,
 /*    80 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   25,
 /*    90 */    25,   38,   41,   42,   43,   44,   45,   46,   47,   48,
 /*   100 */    24,   61,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   110 */    34,   18,   36,   37,   25,   25,   40,   41,   25,   43,
 /*   120 */    44,   45,   46,   47,   48,   24,   64,   26,   27,   28,
 /*   130 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   25,
 /*   140 */    25,   25,   41,   42,   43,   44,   45,   46,   47,   48,
 /*   150 */    24,   30,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   160 */    34,   18,   36,   37,   25,   25,   25,   41,   25,   43,
 /*   170 */    44,   45,   46,   47,   48,   24,   30,   26,   27,   28,
 /*   180 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   25,
 /*   190 */    25,   25,   41,   35,   43,   44,   45,   46,   47,   48,
 /*   200 */    24,   63,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   210 */    34,   25,   36,   37,   25,   25,   25,   41,   50,   43,
 /*   220 */    44,   45,   46,   47,   48,   24,   53,   26,   27,   28,
 /*   230 */    29,   63,   31,   32,   33,   34,   18,   36,   37,   25,
 /*   240 */    11,   25,   41,   25,   43,   44,   45,   46,   47,   48,
 /*   250 */    24,   55,   26,   27,   28,   29,   25,   31,   32,   33,
 /*   260 */    34,   50,   36,   37,   25,   25,   25,   41,   25,   43,
 /*   270 */    44,   45,   46,   47,   48,   24,   67,   26,   27,   28,
 /*   280 */    29,   25,   31,   32,   33,   34,   25,   36,   37,   67,
 /*   290 */    67,   67,   41,   85,   43,   44,   45,   46,   47,   48,
 /*   300 */    24,   67,   26,   27,   28,   29,   94,   31,   32,   33,
 /*   310 */    34,   67,   36,   37,   67,   67,   67,   41,   90,   43,
 /*   320 */    44,   45,   46,   47,   48,   24,   67,   26,   27,   28,
 /*   330 */    29,   67,   31,   32,   33,   34,   67,   36,   37,   67,
 /*   340 */    67,   67,   41,   94,   43,   44,   45,   46,   47,   48,
 /*   350 */    24,   67,   26,   27,   28,   29,   67,   31,   32,   33,
 /*   360 */    34,   67,   36,   37,   67,   67,   67,   41,   94,   43,
 /*   370 */    44,   45,   46,   47,   48,   24,   67,   26,   27,   28,
 /*   380 */    29,   94,   31,   32,   33,   34,   94,   36,   37,   94,
 /*   390 */    94,   94,   41,   94,   43,   44,   45,   46,   47,   48,
 /*   400 */    24,   94,   26,   27,   28,   29,   94,   31,   32,   33,
 /*   410 */    34,   94,   36,   37,   94,   94,   94,   41,   94,   43,
 /*   420 */    44,   45,   46,   47,   48,   24,   94,   26,   27,   28,
 /*   430 */    29,   94,   31,   32,   33,   34,   94,   36,   37,   94,
 /*   440 */    94,   94,   41,   94,   43,   44,   45,   46,   47,   48,
 /*   450 */    24,   94,   26,   27,   28,   29,   13,   31,   32,   33,
 /*   460 */    34,   94,   36,   37,   94,   94,   94,   41,   25,   43,
 /*   470 */    44,   45,   46,   47,   48,    3,    4,    5,    6,    7,
 /*   480 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*   490 */    18,   19,   24,   94,   26,   27,   28,   29,   94,   31,
 /*   500 */    32,   33,   34,   94,   36,   37,   18,   64,   25,   41,
 /*   510 */    94,   43,   44,   45,   46,   47,   48,   57,   58,   59,
 /*   520 */    60,   24,   39,   26,   27,   28,   29,   55,   31,   32,
 /*   530 */    33,   34,   94,   36,   37,   94,   94,   49,   41,   94,
 /*   540 */    43,   44,   45,   46,   47,   48,    3,    4,    5,    6,
 /*   550 */     7,    8,    9,   10,   11,   12,   13,   14,   15,   16,
 /*   560 */    17,   18,   19,   15,   16,   17,   18,   19,   25,   24,
 /*   570 */    94,   26,   27,   28,   29,   94,   31,   32,   33,   34,
 /*   580 */    94,   36,   37,   94,   94,   94,   41,   70,   43,   44,
 /*   590 */    45,   46,   47,   48,   24,   94,   26,   27,   81,   29,
 /*   600 */    94,   31,   32,   33,   34,   94,   36,   37,   91,   92,
 /*   610 */    93,   41,   94,   43,   44,   45,   46,   47,   48,    3,
 /*   620 */     4,    5,    6,    7,    8,    9,   10,   11,   12,   13,
 /*   630 */    14,   15,   16,   17,   18,   19,    4,    5,    6,    7,
 /*   640 */     8,    9,   10,   11,   12,   13,   14,   15,   16,   17,
 /*   650 */    18,   19,    5,    6,    7,    8,    9,   10,   11,   12,
 /*   660 */    13,   14,   15,   16,   17,   18,   19,   13,   28,   29,
 /*   670 */     2,   31,   94,   94,   94,   94,   94,   94,   81,   25,
 /*   680 */    94,   13,   28,   29,   30,   31,   46,   94,   91,   92,
 /*   690 */    93,   94,   38,   94,   94,   94,   28,   29,   94,   31,
 /*   700 */    46,   81,   62,   94,   94,   51,   52,   53,   54,   94,
 /*   710 */    90,   13,   92,   93,   46,   94,   62,   94,   64,   51,
 /*   720 */    52,   53,   54,   25,   56,   70,   28,   29,   30,   31,
 /*   730 */    62,   94,   64,   94,   94,   94,   81,   94,   94,   94,
 /*   740 */    94,   13,   87,   88,   46,   90,   91,   92,   93,   51,
 /*   750 */    52,   53,   54,   25,   71,   94,   28,   29,   94,   31,
 /*   760 */    62,   94,   64,   94,   81,   82,   38,   94,   85,   94,
 /*   770 */    94,   13,   94,   90,   46,   92,   93,   94,   94,   51,
 /*   780 */    52,   53,   54,   25,   94,   94,   28,   29,   81,   31,
 /*   790 */    62,   94,   64,   94,   81,   88,   38,   90,   94,   92,
 /*   800 */    93,   13,   94,   81,   46,   92,   93,   94,   94,   51,
 /*   810 */    52,   53,   54,   25,   92,   93,   28,   29,   94,   31,
 /*   820 */    62,   69,   64,   94,   72,   73,   74,   75,   76,   77,
 /*   830 */    78,   79,   80,   13,   46,   94,   84,   94,   86,   51,
 /*   840 */    52,   53,   54,   94,   81,   25,   94,   13,   28,   29,
 /*   850 */    62,   31,   64,   94,   25,   92,   93,   28,   29,   94,
 /*   860 */    31,   13,   28,   29,   94,   31,   46,   94,   94,   94,
 /*   870 */    94,   51,   52,   53,   54,   46,   28,   29,   81,   31,
 /*   880 */    46,   81,   62,   94,   64,   51,   52,   53,   54,   92,
 /*   890 */    93,   62,   92,   93,   46,   81,   62,   81,   64,   51,
 /*   900 */    52,   53,   54,   70,   94,   94,   92,   93,   92,   93,
 /*   910 */    62,   94,   64,   94,   81,   94,   83,   94,   85,   70,
 /*   920 */    94,   94,   89,   90,   91,   92,   93,   94,   94,   94,
 /*   930 */    81,   94,   83,   70,   85,   94,   94,   94,   89,   90,
 /*   940 */    91,   92,   93,   94,   81,   94,   83,   70,   85,   94,
 /*   950 */    70,   94,   89,   90,   91,   92,   93,   94,   81,   94,
 /*   960 */    83,   81,   85,   81,   94,   70,   89,   90,   91,   92,
 /*   970 */    93,   91,   92,   93,   92,   93,   81,   94,   83,   81,
 /*   980 */    85,   70,   94,   94,   89,   90,   91,   92,   93,   94,
 /*   990 */    92,   93,   81,   94,   83,   70,   85,   94,   94,   94,
 /*  1000 */    89,   90,   91,   92,   93,   94,   81,   94,   83,   70,
 /*  1010 */    85,   94,   94,   94,   89,   90,   91,   92,   93,   94,
 /*  1020 */    81,   94,   83,   70,   85,   94,   94,   94,   89,   90,
 /*  1030 */    91,   92,   93,   94,   81,   94,   83,   70,   85,   94,
 /*  1040 */    94,   94,   89,   90,   91,   92,   93,   94,   81,   94,
 /*  1050 */    83,   70,   85,   94,   94,   94,   89,   90,   91,   92,
 /*  1060 */    93,   94,   81,   94,   94,   70,   85,   94,   94,   94,
 /*  1070 */    89,   90,   91,   92,   93,   94,   81,   94,   94,   70,
 /*  1080 */    85,   94,   94,   94,   89,   90,   91,   92,   93,   94,
 /*  1090 */    81,   94,   94,   70,   85,   94,   94,   94,   89,   90,
 /*  1100 */    91,   92,   93,   94,   81,   94,   81,   13,   85,   94,
 /*  1110 */    71,   94,   89,   90,   91,   92,   93,   92,   93,   94,
 /*  1120 */    81,   82,   28,   29,   85,   31,   94,   94,   71,   90,
 /*  1130 */    94,   92,   93,   81,   94,   94,   94,   94,   81,   82,
 /*  1140 */    46,   94,   85,   71,   92,   93,   94,   90,   94,   92,
 /*  1150 */    93,   94,   94,   81,   94,   94,   62,   85,   64,   71,
 /*  1160 */    94,   94,   90,   94,   92,   93,   94,   94,   94,   81,
 /*  1170 */    94,   94,   71,   85,   94,   94,   94,   94,   90,   94,
 /*  1180 */    92,   93,   81,   94,   94,   71,   85,   94,   94,   94,
 /*  1190 */    94,   90,   94,   92,   93,   81,   94,    1,   94,   85,
 /*  1200 */    71,   94,   94,   94,   90,   94,   92,   93,   94,   94,
 /*  1210 */    81,   94,   94,   94,   85,   71,   20,   21,   22,   90,
 /*  1220 */    94,   92,   93,   94,   70,   81,   94,   94,   70,   85,
 /*  1230 */    94,   81,   94,    1,   90,   81,   92,   93,   88,   81,
 /*  1240 */    90,    1,   92,   93,    1,   91,   92,   93,    1,   91,
 /*  1250 */    92,   93,   20,   21,   22,    1,   94,   94,   94,   94,
 /*  1260 */    20,   21,   22,   20,   21,   22,    1,   20,   21,   22,
 /*  1270 */     1,   94,   94,   94,   20,   21,   22,    1,   94,   94,
 /*  1280 */    94,   94,   94,   94,   94,   20,   21,   22,    1,   20,
 /*  1290 */    21,   22,    1,   94,   94,   94,   20,   21,   22,    1,
 /*  1300 */    94,   94,   94,   94,   94,   94,   94,   20,   21,   22,
 /*  1310 */     1,   20,   21,   22,    1,   94,   94,   94,   20,   21,
 /*  1320 */    22,    1,   94,   94,   94,   94,   94,   94,   94,   20,
 /*  1330 */    21,   22,    1,   20,   21,   22,    1,   94,   94,   94,
 /*  1340 */    20,   21,   22,    1,   94,   94,   94,   94,   94,   94,
 /*  1350 */    94,   20,   21,   22,    1,   20,   21,   22,    1,   94,
 /*  1360 */    94,   94,   20,   21,   22,    1,   94,   94,   94,   94,
 /*  1370 */    94,   94,   94,   20,   21,   22,    1,   20,   21,   22,
 /*  1380 */     1,   94,   94,   94,   20,   21,   22,   94,   94,   94,
 /*  1390 */    94,   94,   94,   94,   94,   20,   21,   22,   94,   20,
 /*  1400 */    21,   22,
);
    const YY_SHIFT_USE_DFLT = -25;
    const YY_SHIFT_MAX = 187;
    static public $yy_shift_ofst = array(
 /*     0 */   -25,  101,   51,  -24,   76,    1,   26,  468,  426,  351,
 /*    10 */   201,  176,  126,  276,  326,  401,  251,  545,  301,  376,
 /*    20 */   151,  497,  226,  570,  668,  668,  668,  668,  668,  668,
 /*    30 */   668,  668,  668,  668,  848,  848,  848,  848, 1094,  698,
 /*    40 */   788,  820,  654,  728,  758,  834,  834,  834,  834,  834,
 /*    50 */  1094, 1094,  640,  640,  640,  640, 1094,  640,  829,  640,
 /*    60 */   640,  640,  640,  640,  640,  640,  640,  640,  640,  640,
 /*    70 */   640, 1357, 1375, 1364, 1379, 1353, 1313, 1254, 1265, 1243,
 /*    80 */  1232, 1269, 1320, 1309, 1291, 1342, 1298, 1287, 1331, 1196,
 /*    90 */  1276, 1247, 1240, 1335,  443,  -14,  -25,  -25,  -25,  -25,
 /*   100 */   -25,  -25,  -25,  -25,  -25,  -25,  -25,  -25,  -25,  -25,
 /*   110 */   -25,  -25,  -25,  -25,  -25,  -25,  -25,  -25,  472,  543,
 /*   120 */   616,  616,  632,  647,  647,  548,  460,  143,  488,  483,
 /*   130 */    -4,   93,   53,  -17,  168,  218,   31,   16,   11,    6,
 /*   140 */    56,   61,   90,   89,   81,   65,  141,  231,  229,  191,
 /*   150 */   173,  239,  256,  243,  196,  241,  138,  164,  165,   40,
 /*   160 */   156,   62,  121,  189,  186,  166,  181,  116,  146,  158,
 /*   170 */    58,  161,  190,  240,  261,  211,  214,  216,   64,  106,
 /*   180 */   111,   -9,   36,  114,  139,  140,  131,  115,
);
    const YY_REDUCE_USE_DFLT = -68;
    const YY_REDUCE_MAX = 117;
    static public $yy_reduce_ofst = array(
 /*     0 */    -1,  752,  752,  752,  752,  752,  752,  752,  752,  752,
 /*    10 */   752,  752,  752,  752,  752,  752,  752,  752,  752,  752,
 /*    20 */   752,  752,  752,  752,  895,  911,  925,  877,  953,  863,
 /*    30 */   833,  849,  939,  967, 1023, 1009,  995,  981,  655,  683,
 /*    40 */  1057, 1039, 1072, 1072, 1072, 1144, 1114, 1129, 1088, 1101,
 /*    50 */   707, 1150,  880, 1154,  517, 1158,  620,  597,  814,  713,
 /*    60 */  1052,  763,  816,  882,  797,  722,  800,  898, 1025,  -42,
 /*    70 */   -67,  -57,  -57,  -57,  -57,  -57,  -57,  -57,  -57,  -57,
 /*    80 */   -57,  -57,  -57,  -57,  -57,  -57,  -57,  -57,  -57,  -57,
 /*    90 */   -57,  -57,  -57,  -57,  228,  208,  274,  284,  273,  272,
 /*   100 */   234,  224,  223,  209,  222,  244,  247,  294,  269,  264,
 /*   110 */   259,  248,  249,  289,  297,  309,  298,  299,
);
    static public $yyExpectedTokens = array(
        /* 0 */ array(),
        /* 1 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 2 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 3 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 4 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 40, 41, 43, 44, 45, 46, 47, 48, ),
        /* 5 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 6 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 42, 43, 44, 45, 46, 47, 48, ),
        /* 7 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 8 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 9 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 10 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 11 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 12 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 13 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 14 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 15 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 16 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 17 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 18 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 19 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 20 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 21 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 22 */ array(24, 26, 27, 28, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 23 */ array(24, 26, 27, 29, 31, 32, 33, 34, 36, 37, 41, 43, 44, 45, 46, 47, 48, ),
        /* 24 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 25 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 26 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 27 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 28 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 29 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 30 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 31 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 32 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 33 */ array(2, 13, 28, 29, 31, 46, 51, 52, 53, 54, 56, 62, 64, ),
        /* 34 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 35 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 36 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 37 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 38 */ array(13, 28, 29, 31, 46, 62, 64, ),
        /* 39 */ array(13, 25, 28, 29, 30, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 40 */ array(13, 25, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 41 */ array(13, 25, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 42 */ array(13, 25, 28, 29, 30, 31, 38, 46, 51, 52, 53, 54, 62, 64, ),
        /* 43 */ array(13, 25, 28, 29, 31, 38, 46, 51, 52, 53, 54, 62, 64, ),
        /* 44 */ array(13, 25, 28, 29, 31, 38, 46, 51, 52, 53, 54, 62, 64, ),
        /* 45 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 46 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 47 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 48 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 49 */ array(13, 28, 29, 31, 46, 51, 52, 53, 54, 62, 64, ),
        /* 50 */ array(13, 28, 29, 31, 46, 62, 64, ),
        /* 51 */ array(13, 28, 29, 31, 46, 62, 64, ),
        /* 52 */ array(28, 29, 31, 46, 62, ),
        /* 53 */ array(28, 29, 31, 46, 62, ),
        /* 54 */ array(28, 29, 31, 46, 62, ),
        /* 55 */ array(28, 29, 31, 46, 62, ),
        /* 56 */ array(13, 28, 29, 31, 46, 62, 64, ),
        /* 57 */ array(28, 29, 31, 46, 62, ),
        /* 58 */ array(25, 28, 29, 31, 46, 62, ),
        /* 59 */ array(28, 29, 31, 46, 62, ),
        /* 60 */ array(28, 29, 31, 46, 62, ),
        /* 61 */ array(28, 29, 31, 46, 62, ),
        /* 62 */ array(28, 29, 31, 46, 62, ),
        /* 63 */ array(28, 29, 31, 46, 62, ),
        /* 64 */ array(28, 29, 31, 46, 62, ),
        /* 65 */ array(28, 29, 31, 46, 62, ),
        /* 66 */ array(28, 29, 31, 46, 62, ),
        /* 67 */ array(28, 29, 31, 46, 62, ),
        /* 68 */ array(28, 29, 31, 46, 62, ),
        /* 69 */ array(28, 29, 31, 46, 62, ),
        /* 70 */ array(28, 29, 31, 46, 62, ),
        /* 71 */ array(1, 20, 21, 22, ),
        /* 72 */ array(1, 20, 21, 22, ),
        /* 73 */ array(1, 20, 21, 22, ),
        /* 74 */ array(1, 20, 21, 22, ),
        /* 75 */ array(1, 20, 21, 22, ),
        /* 76 */ array(1, 20, 21, 22, ),
        /* 77 */ array(1, 20, 21, 22, ),
        /* 78 */ array(1, 20, 21, 22, ),
        /* 79 */ array(1, 20, 21, 22, ),
        /* 80 */ array(1, 20, 21, 22, ),
        /* 81 */ array(1, 20, 21, 22, ),
        /* 82 */ array(1, 20, 21, 22, ),
        /* 83 */ array(1, 20, 21, 22, ),
        /* 84 */ array(1, 20, 21, 22, ),
        /* 85 */ array(1, 20, 21, 22, ),
        /* 86 */ array(1, 20, 21, 22, ),
        /* 87 */ array(1, 20, 21, 22, ),
        /* 88 */ array(1, 20, 21, 22, ),
        /* 89 */ array(1, 20, 21, 22, ),
        /* 90 */ array(1, 20, 21, 22, ),
        /* 91 */ array(1, 20, 21, 22, ),
        /* 92 */ array(1, 20, 21, 22, ),
        /* 93 */ array(1, 20, 21, 22, ),
        /* 94 */ array(13, 25, 64, ),
        /* 95 */ array(53, 54, ),
        /* 96 */ array(),
        /* 97 */ array(),
        /* 98 */ array(),
        /* 99 */ array(),
        /* 100 */ array(),
        /* 101 */ array(),
        /* 102 */ array(),
        /* 103 */ array(),
        /* 104 */ array(),
        /* 105 */ array(),
        /* 106 */ array(),
        /* 107 */ array(),
        /* 108 */ array(),
        /* 109 */ array(),
        /* 110 */ array(),
        /* 111 */ array(),
        /* 112 */ array(),
        /* 113 */ array(),
        /* 114 */ array(),
        /* 115 */ array(),
        /* 116 */ array(),
        /* 117 */ array(),
        /* 118 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 55, ),
        /* 119 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 25, ),
        /* 120 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 121 */ array(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 122 */ array(4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 123 */ array(5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 124 */ array(5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ),
        /* 125 */ array(15, 16, 17, 18, 19, ),
        /* 126 */ array(57, 58, 59, 60, ),
        /* 127 */ array(18, 25, ),
        /* 128 */ array(18, 49, ),
        /* 129 */ array(25, 39, ),
        /* 130 */ array(18, 19, ),
        /* 131 */ array(18, 25, ),
        /* 132 */ array(11, 38, ),
        /* 133 */ array(18, 23, ),
        /* 134 */ array(50, 63, ),
        /* 135 */ array(18, 25, ),
        /* 136 */ array(25, ),
        /* 137 */ array(25, ),
        /* 138 */ array(25, ),
        /* 139 */ array(25, ),
        /* 140 */ array(25, ),
        /* 141 */ array(25, ),
        /* 142 */ array(25, ),
        /* 143 */ array(25, ),
        /* 144 */ array(25, ),
        /* 145 */ array(25, ),
        /* 146 */ array(25, ),
        /* 147 */ array(25, ),
        /* 148 */ array(11, ),
        /* 149 */ array(25, ),
        /* 150 */ array(53, ),
        /* 151 */ array(25, ),
        /* 152 */ array(25, ),
        /* 153 */ array(25, ),
        /* 154 */ array(55, ),
        /* 155 */ array(25, ),
        /* 156 */ array(63, ),
        /* 157 */ array(25, ),
        /* 158 */ array(25, ),
        /* 159 */ array(61, ),
        /* 160 */ array(25, ),
        /* 161 */ array(64, ),
        /* 162 */ array(30, ),
        /* 163 */ array(25, ),
        /* 164 */ array(25, ),
        /* 165 */ array(25, ),
        /* 166 */ array(25, ),
        /* 167 */ array(25, ),
        /* 168 */ array(30, ),
        /* 169 */ array(35, ),
        /* 170 */ array(18, ),
        /* 171 */ array(25, ),
        /* 172 */ array(25, ),
        /* 173 */ array(25, ),
        /* 174 */ array(25, ),
        /* 175 */ array(50, ),
        /* 176 */ array(25, ),
        /* 177 */ array(25, ),
        /* 178 */ array(25, ),
        /* 179 */ array(25, ),
        /* 180 */ array(25, ),
        /* 181 */ array(25, ),
        /* 182 */ array(25, ),
        /* 183 */ array(25, ),
        /* 184 */ array(25, ),
        /* 185 */ array(25, ),
        /* 186 */ array(25, ),
        /* 187 */ array(25, ),
        /* 188 */ array(),
        /* 189 */ array(),
        /* 190 */ array(),
        /* 191 */ array(),
        /* 192 */ array(),
        /* 193 */ array(),
        /* 194 */ array(),
        /* 195 */ array(),
        /* 196 */ array(),
        /* 197 */ array(),
        /* 198 */ array(),
        /* 199 */ array(),
        /* 200 */ array(),
        /* 201 */ array(),
        /* 202 */ array(),
        /* 203 */ array(),
        /* 204 */ array(),
        /* 205 */ array(),
        /* 206 */ array(),
        /* 207 */ array(),
        /* 208 */ array(),
        /* 209 */ array(),
        /* 210 */ array(),
        /* 211 */ array(),
        /* 212 */ array(),
        /* 213 */ array(),
        /* 214 */ array(),
        /* 215 */ array(),
        /* 216 */ array(),
        /* 217 */ array(),
        /* 218 */ array(),
        /* 219 */ array(),
        /* 220 */ array(),
        /* 221 */ array(),
        /* 222 */ array(),
        /* 223 */ array(),
        /* 224 */ array(),
        /* 225 */ array(),
        /* 226 */ array(),
        /* 227 */ array(),
        /* 228 */ array(),
        /* 229 */ array(),
        /* 230 */ array(),
        /* 231 */ array(),
        /* 232 */ array(),
        /* 233 */ array(),
        /* 234 */ array(),
        /* 235 */ array(),
        /* 236 */ array(),
        /* 237 */ array(),
        /* 238 */ array(),
        /* 239 */ array(),
        /* 240 */ array(),
        /* 241 */ array(),
        /* 242 */ array(),
        /* 243 */ array(),
        /* 244 */ array(),
        /* 245 */ array(),
        /* 246 */ array(),
        /* 247 */ array(),
        /* 248 */ array(),
        /* 249 */ array(),
        /* 250 */ array(),
        /* 251 */ array(),
        /* 252 */ array(),
        /* 253 */ array(),
        /* 254 */ array(),
        /* 255 */ array(),
        /* 256 */ array(),
        /* 257 */ array(),
        /* 258 */ array(),
        /* 259 */ array(),
        /* 260 */ array(),
        /* 261 */ array(),
        /* 262 */ array(),
        /* 263 */ array(),
        /* 264 */ array(),
);
    static public $yy_default = array(
 /*     0 */   267,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    10 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    20 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    30 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    40 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    50 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    60 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    70 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*    80 */   354,  354,  354,  354,  354,  354,  265,  354,  354,  354,
 /*    90 */   354,  354,  354,  354,  354,  354,  267,  267,  267,  267,
 /*   100 */   267,  267,  267,  267,  267,  267,  267,  267,  267,  267,
 /*   110 */   267,  267,  267,  267,  267,  267,  267,  267,  354,  354,
 /*   120 */   292,  334,  335,  336,  338,  337,  343,  354,  354,  354,
 /*   130 */   339,  354,  354,  354,  320,  354,  354,  354,  354,  354,
 /*   140 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*   150 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*   160 */   354,  354,  354,  354,  354,  354,  354,  354,  354,  354,
 /*   170 */   328,  354,  354,  354,  354,  320,  354,  354,  354,  354,
 /*   180 */   354,  354,  354,  354,  354,  354,  354,  354,  271,  289,
 /*   190 */   296,  306,  313,  270,  315,  299,  351,  316,  269,  304,
 /*   200 */   298,  350,  349,  314,  297,  276,  277,  275,  274,  353,
 /*   210 */   273,  278,  279,  284,  285,  283,  282,  280,  281,  352,
 /*   220 */   347,  325,  326,  324,  272,  266,  268,  327,  332,  345,
 /*   230 */   346,  348,  344,  333,  286,  287,  301,  302,  300,  295,
 /*   240 */   293,  294,  303,  305,  310,  311,  309,  308,  307,  342,
 /*   250 */   341,  290,  291,  288,  323,  321,  322,  317,  319,  331,
 /*   260 */   340,  330,  329,  318,  312,
);
/* The next thing included is series of defines which control
** various aspects of the generated parser.
**    self::YYNOCODE      is a number which corresponds
**                        to no legal terminal or nonterminal number.  This
**                        number is used to fill in empty slots of the hash 
**                        table.
**    self::YYFALLBACK    If defined, this indicates that one or more tokens
**                        have fall-back values which should be used if the
**                        original value of the token will not parse.
**    self::YYSTACKDEPTH  is the maximum depth of the parser's stack.
**    self::YYNSTATE      the combined number of states.
**    self::YYNRULE       the number of rules in the grammar
**    self::YYERRORSYMBOL is the code number of the error symbol.  If not
**                        defined, then do no error processing.
*/
    const YYNOCODE = 95;
    const YYSTACKDEPTH = 100;
    const YYNSTATE = 265;
    const YYNRULE = 89;
    const YYERRORSYMBOL = 65;
    const YYERRSYMDT = 'yy0';
    const YYFALLBACK = 0;
    /** The next table maps tokens into fallback tokens.  If a construct
     * like the following:
     * 
     *      %fallback ID X Y Z.
     *
     * appears in the grammer, then ID becomes a fallback token for X, Y,
     * and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
     * but it does not parse, the type of the token is changed to ID and
     * the parse is retried before an error is thrown.
     */
    static public $yyFallback = array(
    );
    /**
     * Turn parser tracing on by giving a stream to which to write the trace
     * and a prompt to preface each trace message.  Tracing is turned off
     * by making either argument NULL 
     *
     * Inputs:
     * 
     * - A stream resource to which trace output should be written.
     *   If NULL, then tracing is turned off.
     * - A prefix string written at the beginning of every
     *   line of trace output.  If NULL, then tracing is
     *   turned off.
     *
     * Outputs:
     * 
     * - None.
     * @param resource
     * @param string
     */
    static function Trace($TraceFILE, $zTracePrompt)
    {
        if (!$TraceFILE) {
            $zTracePrompt = 0;
        } elseif (!$zTracePrompt) {
            $TraceFILE = 0;
        }
        self::$yyTraceFILE = $TraceFILE;
        self::$yyTracePrompt = $zTracePrompt;
    }

    /**
     * Output debug information to output (php://output stream)
     */
    static function PrintTrace()
    {
        self::$yyTraceFILE = fopen('php://output', 'w');
        self::$yyTracePrompt = '';
    }

    /**
     * @var resource|0
     */
    static public $yyTraceFILE;
    /**
     * String to prepend to debug output
     * @var string|0
     */
    static public $yyTracePrompt;
    /**
     * @var int
     */
    public $yyidx = -1;                    /* Index of top element in stack */
    /**
     * @var int
     */
    public $yyerrcnt;                 /* Shifts left before out of the error */
    /**
     * @var array
     */
    public $yystack = array();  /* The parser's stack */

    /**
     * For tracing shifts, the names of all terminals and nonterminals
     * are required.  The following table supplies these names
     * @var array
     */
    static public $yyTokenName = array( 
  '$',             'T_TAG_OPEN',    'T_NOT',         'T_AND',       
  'T_OR',          'T_EQ',          'T_NE',          'T_GT',        
  'T_GE',          'T_LT',          'T_LE',          'T_IN',        
  'T_PLUS',        'T_MINUS',       'T_CONCAT',      'T_TIMES',     
  'T_DIV',         'T_MOD',         'T_PIPE',        'T_BITWISE',   
  'T_HTML',        'T_COMMENT',     'T_PRINT_OPEN',  'T_PRINT_CLOSE',
  'T_EXTENDS',     'T_TAG_CLOSE',   'T_INCLUDE',     'T_AUTOESCAPE',
  'T_CUSTOM_END',  'T_CUSTOM_TAG',  'T_AS',          'T_CUSTOM_BLOCK',
  'T_SPACEFULL',   'T_WITH',        'T_SET',         'T_ASSIGN',    
  'T_LOAD',        'T_FOR',         'T_COMMA',       'T_STEP',      
  'T_EMPTY',       'T_IF',          'T_ELSE',        'T_IFCHANGED', 
  'T_IFEQUAL',     'T_IFNOTEQUAL',  'T_BLOCK',       'T_FILTER',    
  'T_REGROUP',     'T_BY',          'T_COLON',       'T_TRUE',      
  'T_FALSE',       'T_STRING',      'T_INTL',        'T_RPARENT',   
  'T_LPARENT',     'T_OBJ',         'T_DOT',         'T_CLASS',     
  'T_BRACKETS_OPEN',  'T_BRACKETS_CLOSE',  'T_ALPHA',       'T_DOTDOT',    
  'T_NUMERIC',     'error',         'start',         'body',        
  'code',          'stmts',         'filtered_var',  'var_or_string',
  'stmt',          'for_stmt',      'ifchanged_stmt',  'block_stmt',  
  'filter_stmt',   'if_stmt',       'custom_tag',    'alias',       
  'ifequal',       'varname',       'params',        'expr',        
  'regroup',       'string',        'for_def',       'range',       
  'numvar',        'fvar_or_string',  'number',        'varname_args',
  'varpart',       'varpart_single',
    );

    /**
     * For tracing reduce actions, the names of all rules are required.
     * @var array
     */
    static public $yyRuleName = array(
 /*   0 */ "start ::= body",
 /*   1 */ "body ::= body code",
 /*   2 */ "body ::=",
 /*   3 */ "code ::= T_TAG_OPEN stmts",
 /*   4 */ "code ::= T_HTML",
 /*   5 */ "code ::= T_COMMENT",
 /*   6 */ "code ::= T_PRINT_OPEN filtered_var T_PRINT_CLOSE",
 /*   7 */ "stmts ::= T_EXTENDS var_or_string T_TAG_CLOSE",
 /*   8 */ "stmts ::= stmt T_TAG_CLOSE",
 /*   9 */ "stmts ::= for_stmt",
 /*  10 */ "stmts ::= ifchanged_stmt",
 /*  11 */ "stmts ::= block_stmt",
 /*  12 */ "stmts ::= filter_stmt",
 /*  13 */ "stmts ::= if_stmt",
 /*  14 */ "stmts ::= T_INCLUDE var_or_string T_TAG_CLOSE",
 /*  15 */ "stmts ::= custom_tag",
 /*  16 */ "stmts ::= alias",
 /*  17 */ "stmts ::= ifequal",
 /*  18 */ "stmts ::= T_AUTOESCAPE varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  19 */ "custom_tag ::= T_CUSTOM_TAG T_TAG_CLOSE",
 /*  20 */ "custom_tag ::= T_CUSTOM_TAG T_AS varname T_TAG_CLOSE",
 /*  21 */ "custom_tag ::= T_CUSTOM_TAG params T_TAG_CLOSE",
 /*  22 */ "custom_tag ::= T_CUSTOM_TAG params T_AS varname T_TAG_CLOSE",
 /*  23 */ "custom_tag ::= T_CUSTOM_BLOCK T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  24 */ "custom_tag ::= T_CUSTOM_BLOCK params T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  25 */ "custom_tag ::= T_SPACEFULL T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  26 */ "alias ::= T_WITH varname T_AS varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  27 */ "stmt ::= T_SET varname T_ASSIGN expr",
 /*  28 */ "stmt ::= regroup",
 /*  29 */ "stmt ::= T_LOAD string",
 /*  30 */ "for_def ::= T_FOR varname T_IN filtered_var T_TAG_CLOSE",
 /*  31 */ "for_def ::= T_FOR varname T_COMMA varname T_IN filtered_var T_TAG_CLOSE",
 /*  32 */ "for_stmt ::= for_def body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  33 */ "for_stmt ::= T_FOR varname T_IN range T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  34 */ "for_stmt ::= T_FOR varname T_IN range T_STEP numvar T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  35 */ "for_stmt ::= for_def body T_TAG_OPEN T_EMPTY T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  36 */ "if_stmt ::= T_IF expr T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  37 */ "if_stmt ::= T_IF expr T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  38 */ "ifchanged_stmt ::= T_IFCHANGED T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  39 */ "ifchanged_stmt ::= T_IFCHANGED params T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  40 */ "ifchanged_stmt ::= T_IFCHANGED T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  41 */ "ifchanged_stmt ::= T_IFCHANGED params T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  42 */ "ifequal ::= T_IFEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  43 */ "ifequal ::= T_IFEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  44 */ "ifequal ::= T_IFNOTEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  45 */ "ifequal ::= T_IFNOTEQUAL fvar_or_string fvar_or_string T_TAG_CLOSE body T_TAG_OPEN T_ELSE T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  46 */ "block_stmt ::= T_BLOCK varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  47 */ "block_stmt ::= T_BLOCK varname T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END varname T_TAG_CLOSE",
 /*  48 */ "block_stmt ::= T_BLOCK number T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  49 */ "block_stmt ::= T_BLOCK number T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END number T_TAG_CLOSE",
 /*  50 */ "filter_stmt ::= T_FILTER filtered_var T_TAG_CLOSE body T_TAG_OPEN T_CUSTOM_END T_TAG_CLOSE",
 /*  51 */ "regroup ::= T_REGROUP filtered_var T_BY varname T_AS varname",
 /*  52 */ "filtered_var ::= filtered_var T_PIPE varname_args",
 /*  53 */ "filtered_var ::= varname_args",
 /*  54 */ "varname_args ::= varname T_COLON var_or_string",
 /*  55 */ "varname_args ::= varname",
 /*  56 */ "params ::= params var_or_string",
 /*  57 */ "params ::= params T_COMMA var_or_string",
 /*  58 */ "params ::= var_or_string",
 /*  59 */ "var_or_string ::= varname",
 /*  60 */ "var_or_string ::= number",
 /*  61 */ "var_or_string ::= T_TRUE|T_FALSE",
 /*  62 */ "var_or_string ::= string",
 /*  63 */ "fvar_or_string ::= filtered_var",
 /*  64 */ "fvar_or_string ::= number",
 /*  65 */ "fvar_or_string ::= T_TRUE|T_FALSE",
 /*  66 */ "fvar_or_string ::= string",
 /*  67 */ "string ::= T_STRING",
 /*  68 */ "string ::= T_INTL T_STRING T_RPARENT",
 /*  69 */ "expr ::= T_NOT expr",
 /*  70 */ "expr ::= expr T_AND expr",
 /*  71 */ "expr ::= expr T_OR expr",
 /*  72 */ "expr ::= expr T_PLUS|T_MINUS|T_CONCAT expr",
 /*  73 */ "expr ::= expr T_EQ|T_NE|T_GT|T_GE|T_LT|T_LE|T_IN expr",
 /*  74 */ "expr ::= expr T_TIMES|T_DIV|T_MOD expr",
 /*  75 */ "expr ::= expr T_BITWISE|T_PIPE expr",
 /*  76 */ "expr ::= T_LPARENT expr T_RPARENT",
 /*  77 */ "expr ::= fvar_or_string",
 /*  78 */ "varname ::= varpart",
 /*  79 */ "varpart ::= varpart T_OBJ|T_DOT varpart_single",
 /*  80 */ "varpart ::= varpart T_CLASS varpart_single",
 /*  81 */ "varpart ::= varpart T_BRACKETS_OPEN var_or_string T_BRACKETS_CLOSE",
 /*  82 */ "varpart ::= varpart_single",
 /*  83 */ "varpart_single ::= T_ALPHA|T_BLOCK|T_CUSTOM_TAG|T_CUSTOM_END|T_CUSTOM_BLOCK",
 /*  84 */ "range ::= numvar T_DOTDOT numvar",
 /*  85 */ "numvar ::= number",
 /*  86 */ "numvar ::= varname",
 /*  87 */ "number ::= T_NUMERIC",
 /*  88 */ "number ::= T_MINUS T_NUMERIC",
    );

    /**
     * This function returns the symbolic name associated with a token
     * value.
     * @param int
     * @return string
     */
    function tokenName($tokenType)
    {
        if ($tokenType === 0) {
            return 'End of Input';
        }
        if ($tokenType > 0 && $tokenType < count(self::$yyTokenName)) {
            return self::$yyTokenName[$tokenType];
        } else {
            return "Unknown";
        }
    }

    /**
     * The following function deletes the value associated with a
     * symbol.  The symbol can be either a terminal or nonterminal.
     * @param int the symbol code
     * @param mixed the symbol's value
     */
    static function yy_destructor($yymajor, $yypminor)
    {
        switch ($yymajor) {
        /* Here is inserted the actions which take place when a
        ** terminal or non-terminal is destroyed.  This can happen
        ** when the symbol is popped from the stack during a
        ** reduce or during error processing or when a parser is 
        ** being destroyed before it is finished parsing.
        **
        ** Note: during a reduce, the only symbols destroyed are those
        ** which appear on the RHS of the rule, but which are not used
        ** inside the C code.
        */
            default:  break;   /* If no destructor action specified: do nothing */
        }
    }

    /**
     * Pop the parser's stack once.
     *
     * If there is a destructor routine associated with the token which
     * is popped from the stack, then call it.
     *
     * Return the major token number for the symbol popped.
     * @param Haanga_yyParser
     * @return int
     */
    function yy_pop_parser_stack()
    {
        if (!count($this->yystack)) {
            return;
        }
        $yytos = array_pop($this->yystack);
        if (self::$yyTraceFILE && $this->yyidx >= 0) {
            fwrite(self::$yyTraceFILE,
                self::$yyTracePrompt . 'Popping ' . self::$yyTokenName[$yytos->major] .
                    "\n");
        }
        $yymajor = $yytos->major;
        self::yy_destructor($yymajor, $yytos->minor);
        $this->yyidx--;
        return $yymajor;
    }

    /**
     * Deallocate and destroy a parser.  Destructors are all called for
     * all stack elements before shutting the parser down.
     */
    function __destruct()
    {
        while ($this->yyidx >= 0) {
            $this->yy_pop_parser_stack();
        }
        if (is_resource(self::$yyTraceFILE)) {
            fclose(self::$yyTraceFILE);
        }
    }

    /**
     * Based on the current state and parser stack, get a list of all
     * possible lookahead tokens
     * @param int
     * @return array
     */
    function yy_get_expected_tokens($token)
    {
        $state = $this->yystack[$this->yyidx]->stateno;
        $expected = self::$yyExpectedTokens[$state];
        if (in_array($token, self::$yyExpectedTokens[$state], true)) {
            return $expected;
        }
        $stack = $this->yystack;
        $yyidx = $this->yyidx;
        do {
            $yyact = $this->yy_find_shift_action($token);
            if ($yyact >= self::YYNSTATE && $yyact < self::YYNSTATE + self::YYNRULE) {
                // reduce action
                $done = 0;
                do {
                    if ($done++ == 100) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // too much recursion prevents proper detection
                        // so give up
                        return array_unique($expected);
                    }
                    $yyruleno = $yyact - self::YYNSTATE;
                    $this->yyidx -= self::$yyRuleInfo[$yyruleno]['rhs'];
                    $nextstate = $this->yy_find_reduce_action(
                        $this->yystack[$this->yyidx]->stateno,
                        self::$yyRuleInfo[$yyruleno]['lhs']);
                    if (isset(self::$yyExpectedTokens[$nextstate])) {
                        $expected += self::$yyExpectedTokens[$nextstate];
                            if (in_array($token,
                                  self::$yyExpectedTokens[$nextstate], true)) {
                            $this->yyidx = $yyidx;
                            $this->yystack = $stack;
                            return array_unique($expected);
                        }
                    }
                    if ($nextstate < self::YYNSTATE) {
                        // we need to shift a non-terminal
                        $this->yyidx++;
                        $x = new Haanga_yyStackEntry;
                        $x->stateno = $nextstate;
                        $x->major = self::$yyRuleInfo[$yyruleno]['lhs'];
                        $this->yystack[$this->yyidx] = $x;
                        continue 2;
                    } elseif ($nextstate == self::YYNSTATE + self::YYNRULE + 1) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // the last token was just ignored, we can't accept
                        // by ignoring input, this is in essence ignoring a
                        // syntax error!
                        return array_unique($expected);
                    } elseif ($nextstate === self::YY_NO_ACTION) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // input accepted, but not shifted (I guess)
                        return $expected;
                    } else {
                        $yyact = $nextstate;
                    }
                } while (true);
            }
            break;
        } while (true);
        return array_unique($expected);
    }

    /**
     * Based on the parser state and current parser stack, determine whether
     * the lookahead token is possible.
     * 
     * The parser will convert the token value to an error token if not.  This
     * catches some unusual edge cases where the parser would fail.
     * @param int
     * @return bool
     */
    function yy_is_expected_token($token)
    {
        if ($token === 0) {
            return true; // 0 is not part of this
        }
        $state = $this->yystack[$this->yyidx]->stateno;
        if (in_array($token, self::$yyExpectedTokens[$state], true)) {
            return true;
        }
        $stack = $this->yystack;
        $yyidx = $this->yyidx;
        do {
            $yyact = $this->yy_find_shift_action($token);
            if ($yyact >= self::YYNSTATE && $yyact < self::YYNSTATE + self::YYNRULE) {
                // reduce action
                $done = 0;
                do {
                    if ($done++ == 100) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // too much recursion prevents proper detection
                        // so give up
                        return true;
                    }
                    $yyruleno = $yyact - self::YYNSTATE;
                    $this->yyidx -= self::$yyRuleInfo[$yyruleno]['rhs'];
                    $nextstate = $this->yy_find_reduce_action(
                        $this->yystack[$this->yyidx]->stateno,
                        self::$yyRuleInfo[$yyruleno]['lhs']);
                    if (isset(self::$yyExpectedTokens[$nextstate]) &&
                          in_array($token, self::$yyExpectedTokens[$nextstate], true)) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        return true;
                    }
                    if ($nextstate < self::YYNSTATE) {
                        // we need to shift a non-terminal
                        $this->yyidx++;
                        $x = new Haanga_yyStackEntry;
                        $x->stateno = $nextstate;
                        $x->major = self::$yyRuleInfo[$yyruleno]['lhs'];
                        $this->yystack[$this->yyidx] = $x;
                        continue 2;
                    } elseif ($nextstate == self::YYNSTATE + self::YYNRULE + 1) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        if (!$token) {
                            // end of input: this is valid
                            return true;
                        }
                        // the last token was just ignored, we can't accept
                        // by ignoring input, this is in essence ignoring a
                        // syntax error!
                        return false;
                    } elseif ($nextstate === self::YY_NO_ACTION) {
                        $this->yyidx = $yyidx;
                        $this->yystack = $stack;
                        // input accepted, but not shifted (I guess)
                        return true;
                    } else {
                        $yyact = $nextstate;
                    }
                } while (true);
            }
            break;
        } while (true);
        $this->yyidx = $yyidx;
        $this->yystack = $stack;
        return true;
    }

    /**
     * Find the appropriate action for a parser given the terminal
     * look-ahead token iLookAhead.
     *
     * If the look-ahead token is YYNOCODE, then check to see if the action is
     * independent of the look-ahead.  If it is, return the action, otherwise
     * return YY_NO_ACTION.
     * @param int The look-ahead token
     */
    function yy_find_shift_action($iLookAhead)
    {
        $stateno = $this->yystack[$this->yyidx]->stateno;
     
        /* if ($this->yyidx < 0) return self::YY_NO_ACTION;  */
        if (!isset(self::$yy_shift_ofst[$stateno])) {
            // no shift actions
            return self::$yy_default[$stateno];
        }
        $i = self::$yy_shift_ofst[$stateno];
        if ($i === self::YY_SHIFT_USE_DFLT) {
            return self::$yy_default[$stateno];
        }
        if ($iLookAhead == self::YYNOCODE) {
            return self::YY_NO_ACTION;
        }
        $i += $iLookAhead;
        if ($i < 0 || $i >= self::YY_SZ_ACTTAB ||
              self::$yy_lookahead[$i] != $iLookAhead) {
            if (count(self::$yyFallback) && $iLookAhead < count(self::$yyFallback)
                   && ($iFallback = self::$yyFallback[$iLookAhead]) != 0) {
                if (self::$yyTraceFILE) {
                    fwrite(self::$yyTraceFILE, self::$yyTracePrompt . "FALLBACK " .
                        self::$yyTokenName[$iLookAhead] . " => " .
                        self::$yyTokenName[$iFallback] . "\n");
                }
                return $this->yy_find_shift_action($iFallback);
            }
            return self::$yy_default[$stateno];
        } else {
            return self::$yy_action[$i];
        }
    }

    /**
     * Find the appropriate action for a parser given the non-terminal
     * look-ahead token $iLookAhead.
     *
     * If the look-ahead token is self::YYNOCODE, then check to see if the action is
     * independent of the look-ahead.  If it is, return the action, otherwise
     * return self::YY_NO_ACTION.
     * @param int Current state number
     * @param int The look-ahead token
     */
    function yy_find_reduce_action($stateno, $iLookAhead)
    {
        /* $stateno = $this->yystack[$this->yyidx]->stateno; */

        if (!isset(self::$yy_reduce_ofst[$stateno])) {
            return self::$yy_default[$stateno];
        }
        $i = self::$yy_reduce_ofst[$stateno];
        if ($i == self::YY_REDUCE_USE_DFLT) {
            return self::$yy_default[$stateno];
        }
        if ($iLookAhead == self::YYNOCODE) {
            return self::YY_NO_ACTION;
        }
        $i += $iLookAhead;
        if ($i < 0 || $i >= self::YY_SZ_ACTTAB ||
              self::$yy_lookahead[$i] != $iLookAhead) {
            return self::$yy_default[$stateno];
        } else {
            return self::$yy_action[$i];
        }
    }

    /**
     * Perform a shift action.
     * @param int The new state to shift in
     * @param int The major token to shift in
     * @param mixed the minor token to shift in
     */
    function yy_shift($yyNewState, $yyMajor, $yypMinor)
    {
        $this->yyidx++;
        if ($this->yyidx >= self::YYSTACKDEPTH) {
            $this->yyidx--;
            if (self::$yyTraceFILE) {
                fprintf(self::$yyTraceFILE, "%sStack Overflow!\n", self::$yyTracePrompt);
            }
            while ($this->yyidx >= 0) {
                $this->yy_pop_parser_stack();
            }
            /* Here code is inserted which will execute if the parser
            ** stack ever overflows */
            return;
        }
        $yytos = new Haanga_yyStackEntry;
        $yytos->stateno = $yyNewState;
        $yytos->major = $yyMajor;
        $yytos->minor = $yypMinor;
        array_push($this->yystack, $yytos);
        if (self::$yyTraceFILE && $this->yyidx > 0) {
            fprintf(self::$yyTraceFILE, "%sShift %d\n", self::$yyTracePrompt,
                $yyNewState);
            fprintf(self::$yyTraceFILE, "%sStack:", self::$yyTracePrompt);
            for ($i = 1; $i <= $this->yyidx; $i++) {
                fprintf(self::$yyTraceFILE, " %s",
                    self::$yyTokenName[$this->yystack[$i]->major]);
            }
            fwrite(self::$yyTraceFILE,"\n");
        }
    }

    /**
     * The following table contains information about every rule that
     * is used during the reduce.
     *
     * <pre>
     * array(
     *  array(
     *   int $lhs;         Symbol on the left-hand side of the rule
     *   int $nrhs;     Number of right-hand side symbols in the rule
     *  ),...
     * );
     * </pre>
     */
    static public $yyRuleInfo = array(
  array( 'lhs' => 66, 'rhs' => 1 ),
  array( 'lhs' => 67, 'rhs' => 2 ),
  array( 'lhs' => 67, 'rhs' => 0 ),
  array( 'lhs' => 68, 'rhs' => 2 ),
  array( 'lhs' => 68, 'rhs' => 1 ),
  array( 'lhs' => 68, 'rhs' => 1 ),
  array( 'lhs' => 68, 'rhs' => 3 ),
  array( 'lhs' => 69, 'rhs' => 3 ),
  array( 'lhs' => 69, 'rhs' => 2 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 3 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 1 ),
  array( 'lhs' => 69, 'rhs' => 7 ),
  array( 'lhs' => 78, 'rhs' => 2 ),
  array( 'lhs' => 78, 'rhs' => 4 ),
  array( 'lhs' => 78, 'rhs' => 3 ),
  array( 'lhs' => 78, 'rhs' => 5 ),
  array( 'lhs' => 78, 'rhs' => 6 ),
  array( 'lhs' => 78, 'rhs' => 7 ),
  array( 'lhs' => 78, 'rhs' => 6 ),
  array( 'lhs' => 79, 'rhs' => 9 ),
  array( 'lhs' => 72, 'rhs' => 4 ),
  array( 'lhs' => 72, 'rhs' => 1 ),
  array( 'lhs' => 72, 'rhs' => 2 ),
  array( 'lhs' => 86, 'rhs' => 5 ),
  array( 'lhs' => 86, 'rhs' => 7 ),
  array( 'lhs' => 73, 'rhs' => 5 ),
  array( 'lhs' => 73, 'rhs' => 9 ),
  array( 'lhs' => 73, 'rhs' => 11 ),
  array( 'lhs' => 73, 'rhs' => 9 ),
  array( 'lhs' => 77, 'rhs' => 7 ),
  array( 'lhs' => 77, 'rhs' => 11 ),
  array( 'lhs' => 74, 'rhs' => 6 ),
  array( 'lhs' => 74, 'rhs' => 7 ),
  array( 'lhs' => 74, 'rhs' => 10 ),
  array( 'lhs' => 74, 'rhs' => 11 ),
  array( 'lhs' => 80, 'rhs' => 8 ),
  array( 'lhs' => 80, 'rhs' => 12 ),
  array( 'lhs' => 80, 'rhs' => 8 ),
  array( 'lhs' => 80, 'rhs' => 12 ),
  array( 'lhs' => 75, 'rhs' => 7 ),
  array( 'lhs' => 75, 'rhs' => 8 ),
  array( 'lhs' => 75, 'rhs' => 7 ),
  array( 'lhs' => 75, 'rhs' => 8 ),
  array( 'lhs' => 76, 'rhs' => 7 ),
  array( 'lhs' => 84, 'rhs' => 6 ),
  array( 'lhs' => 70, 'rhs' => 3 ),
  array( 'lhs' => 70, 'rhs' => 1 ),
  array( 'lhs' => 91, 'rhs' => 3 ),
  array( 'lhs' => 91, 'rhs' => 1 ),
  array( 'lhs' => 82, 'rhs' => 2 ),
  array( 'lhs' => 82, 'rhs' => 3 ),
  array( 'lhs' => 82, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 71, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 89, 'rhs' => 1 ),
  array( 'lhs' => 85, 'rhs' => 1 ),
  array( 'lhs' => 85, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 2 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 3 ),
  array( 'lhs' => 83, 'rhs' => 1 ),
  array( 'lhs' => 81, 'rhs' => 1 ),
  array( 'lhs' => 92, 'rhs' => 3 ),
  array( 'lhs' => 92, 'rhs' => 3 ),
  array( 'lhs' => 92, 'rhs' => 4 ),
  array( 'lhs' => 92, 'rhs' => 1 ),
  array( 'lhs' => 93, 'rhs' => 1 ),
  array( 'lhs' => 87, 'rhs' => 3 ),
  array( 'lhs' => 88, 'rhs' => 1 ),
  array( 'lhs' => 88, 'rhs' => 1 ),
  array( 'lhs' => 90, 'rhs' => 1 ),
  array( 'lhs' => 90, 'rhs' => 2 ),
    );

    /**
     * The following table contains a mapping of reduce action to method name
     * that handles the reduction.
     * 
     * If a rule is not set, it has no handler.
     */
    static public $yyReduceMap = array(
        0 => 0,
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5,
        6 => 6,
        7 => 7,
        8 => 8,
        68 => 8,
        9 => 9,
        10 => 9,
        11 => 9,
        12 => 9,
        13 => 9,
        15 => 9,
        16 => 9,
        17 => 9,
        28 => 9,
        55 => 9,
        67 => 9,
        77 => 9,
        82 => 9,
        83 => 9,
        85 => 9,
        87 => 9,
        14 => 14,
        18 => 18,
        19 => 19,
        20 => 20,
        21 => 21,
        22 => 22,
        23 => 23,
        24 => 24,
        25 => 25,
        26 => 26,
        27 => 27,
        29 => 29,
        30 => 30,
        31 => 31,
        32 => 32,
        33 => 33,
        34 => 34,
        35 => 35,
        36 => 36,
        37 => 37,
        38 => 38,
        39 => 39,
        40 => 40,
        41 => 41,
        42 => 42,
        43 => 43,
        44 => 44,
        45 => 45,
        46 => 46,
        47 => 47,
        49 => 47,
        48 => 48,
        50 => 50,
        51 => 51,
        52 => 52,
        57 => 52,
        53 => 53,
        58 => 53,
        54 => 54,
        56 => 56,
        59 => 59,
        86 => 59,
        60 => 60,
        64 => 60,
        61 => 61,
        65 => 61,
        62 => 62,
        66 => 62,
        63 => 63,
        69 => 69,
        70 => 70,
        71 => 70,
        72 => 70,
        74 => 70,
        73 => 73,
        75 => 75,
        76 => 76,
        78 => 78,
        79 => 79,
        80 => 80,
        81 => 81,
        84 => 84,
        88 => 88,
    );
    /* Beginning here are the reduction cases.  A typical example
    ** follows:
    **  #line <lineno> <grammarfile>
    **   function yy_r0($yymsp){ ... }           // User supplied code
    **  #line <lineno> <thisfile>
    */
#line 80 "Parser.y"
    function yy_r0(){ $this->body = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1696 "Parser.php"
#line 82 "Parser.y"
    function yy_r1(){ $this->_retvalue=$this->yystack[$this->yyidx + -1]->minor; $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1699 "Parser.php"
#line 83 "Parser.y"
    function yy_r2(){ $this->_retvalue = array();     }
#line 1702 "Parser.php"
#line 86 "Parser.y"
    function yy_r3(){ if (count($this->yystack[$this->yyidx + 0]->minor)) $this->yystack[$this->yyidx + 0]->minor['line'] = $this->lex->getLine();  $this->_retvalue = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1705 "Parser.php"
#line 88 "Parser.y"
    function yy_r4(){
    $this->_retvalue = array('operation' => 'html', 'html' => $this->yystack[$this->yyidx + 0]->minor, 'line' => $this->lex->getLine() ); 
    }
#line 1710 "Parser.php"
#line 92 "Parser.y"
    function yy_r5(){
    $this->yystack[$this->yyidx + 0]->minor=rtrim($this->yystack[$this->yyidx + 0]->minor); $this->_retvalue = array('operation' => 'comment', 'comment' => $this->yystack[$this->yyidx + 0]->minor); 
    }
#line 1715 "Parser.php"
#line 96 "Parser.y"
    function yy_r6(){
    $this->_retvalue = array('operation' => 'print_var', 'variable' => $this->yystack[$this->yyidx + -1]->minor, 'line' => $this->lex->getLine() ); 
    }
#line 1720 "Parser.php"
#line 100 "Parser.y"
    function yy_r7(){ $this->_retvalue = array('operation' => 'base', $this->yystack[$this->yyidx + -1]->minor);     }
#line 1723 "Parser.php"
#line 101 "Parser.y"
    function yy_r8(){ $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor;     }
#line 1726 "Parser.php"
#line 102 "Parser.y"
    function yy_r9(){ $this->_retvalue = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1729 "Parser.php"
#line 107 "Parser.y"
    function yy_r14(){ $this->_retvalue = array('operation' => 'include', $this->yystack[$this->yyidx + -1]->minor);     }
#line 1732 "Parser.php"
#line 111 "Parser.y"
    function yy_r18(){ 
    $this->yystack[$this->yyidx + -5]->minor = strtolower($this->yystack[$this->yyidx + -5]->minor);
    if ($this->yystack[$this->yyidx + -5]->minor != 'on' && $this->yystack[$this->yyidx + -5]->minor != 'off') {
        $this->Error("Invalid autoescape param (".$this->yystack[$this->yyidx + -5]->minor."), it must be on or off");
    }
    if ($this->yystack[$this->yyidx + -1]->minor != "endautoescape") {
        $this->Error("Invalid close tag ".$this->yystack[$this->yyidx + -1]->minor.", it must be endautoescape");
    }
    $this->_retvalue = array('operation' => 'autoescape', 'value' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1744 "Parser.php"
#line 125 "Parser.y"
    function yy_r19(){
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -1]->minor, 'list'=>array()); 
    }
#line 1749 "Parser.php"
#line 128 "Parser.y"
    function yy_r20(){
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -3]->minor, 'as' => $this->yystack[$this->yyidx + -1]->minor, 'list'=>array()); 
    }
#line 1754 "Parser.php"
#line 131 "Parser.y"
    function yy_r21(){ 
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -2]->minor, 'list' => $this->yystack[$this->yyidx + -1]->minor); 
    }
#line 1759 "Parser.php"
#line 134 "Parser.y"
    function yy_r22(){
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -4]->minor, 'as' => $this->yystack[$this->yyidx + -1]->minor, 'list' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1764 "Parser.php"
#line 139 "Parser.y"
    function yy_r23(){
    if ('end'.$this->yystack[$this->yyidx + -5]->minor != $this->yystack[$this->yyidx + -1]->minor) { 
        $this->error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor); 
    } 
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'list' => array());
    }
#line 1772 "Parser.php"
#line 145 "Parser.y"
    function yy_r24(){
    if ('end'.$this->yystack[$this->yyidx + -6]->minor != $this->yystack[$this->yyidx + -1]->minor) { 
        $this->error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor); 
    } 
    $this->_retvalue = array('operation' => 'custom_tag', 'name' => $this->yystack[$this->yyidx + -6]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'list' => $this->yystack[$this->yyidx + -5]->minor);
    }
#line 1780 "Parser.php"
#line 153 "Parser.y"
    function yy_r25(){
    if ('endspacefull' != $this->yystack[$this->yyidx + -1]->minor) {
        $this->error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor);
    } 
    $this->_retvalue = array('operation' => 'spacefull', 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1788 "Parser.php"
#line 161 "Parser.y"
    function yy_r26(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endwith") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endwith");
    }
    $this->_retvalue = array('operation' => 'alias', 'var' => $this->yystack[$this->yyidx + -7]->minor, 'as' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1796 "Parser.php"
#line 169 "Parser.y"
    function yy_r27(){ $this->_retvalue = array('operation' => 'set', 'var' => $this->yystack[$this->yyidx + -2]->minor,'expr' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1799 "Parser.php"
#line 171 "Parser.y"
    function yy_r29(){
    if (!is_file($this->yystack[$this->yyidx + 0]->minor) || !Haanga_Compiler::getOption('enable_load')) {
        $this->error($this->yystack[$this->yyidx + 0]->minor." is not a valid file"); 
    } 
    require_once $this->yystack[$this->yyidx + 0]->minor;
    }
#line 1807 "Parser.php"
#line 180 "Parser.y"
    function yy_r30(){
    $var = $this->compiler->get_context($this->yystack[$this->yyidx + -1]->minor[0]);
    if (is_array($var) || $var instanceof Iterator) {
        /* let's check if it is an object or array */
        $this->compiler->set_context($this->yystack[$this->yyidx + -3]->minor, current($var));
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -3]->minor, 'index' => NULL, 'array' => $this->yystack[$this->yyidx + -1]->minor);
    }
#line 1817 "Parser.php"
#line 188 "Parser.y"
    function yy_r31(){
    $var = $this->compiler->get_context($this->yystack[$this->yyidx + -1]->minor[0]);
    if (is_array($var) || $var instanceof Iterator) {
        /* let's check if it is an object or array */
        $this->compiler->set_context($this->yystack[$this->yyidx + -3]->minor, current($var));
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -3]->minor, 'index' => $this->yystack[$this->yyidx + -5]->minor, 'array' => $this->yystack[$this->yyidx + -1]->minor);

    }
#line 1828 "Parser.php"
#line 198 "Parser.y"
    function yy_r32(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = $this->yystack[$this->yyidx + -4]->minor;
    $this->_retvalue['body'] = $this->yystack[$this->yyidx + -3]->minor;
    }
#line 1837 "Parser.php"
#line 206 "Parser.y"
    function yy_r33(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -7]->minor, 'range' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'variable' => $this->yystack[$this->yyidx + -7]->minor, 'step' => 1);
    }
#line 1845 "Parser.php"
#line 213 "Parser.y"
    function yy_r34(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = array('operation' => 'loop', 'variable' => $this->yystack[$this->yyidx + -9]->minor, 'range' => $this->yystack[$this->yyidx + -7]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor, 'variable' => $this->yystack[$this->yyidx + -9]->minor, 'step' => $this->yystack[$this->yyidx + -5]->minor);
    }
#line 1853 "Parser.php"
#line 220 "Parser.y"
    function yy_r35(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endfor") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfor");
    }
    $this->_retvalue = $this->yystack[$this->yyidx + -8]->minor;
    $this->_retvalue['body']  = $this->yystack[$this->yyidx + -7]->minor;
    $this->_retvalue['empty'] = $this->yystack[$this->yyidx + -3]->minor;
    }
#line 1863 "Parser.php"
#line 229 "Parser.y"
    function yy_r36(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endif") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endif");
    }
    $this->_retvalue = array('operation' => 'if', 'expr' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1871 "Parser.php"
#line 235 "Parser.y"
    function yy_r37(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endif") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endif");
    }
    $this->_retvalue = array('operation' => 'if', 'expr' => $this->yystack[$this->yyidx + -9]->minor, 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1879 "Parser.php"
#line 243 "Parser.y"
    function yy_r38(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1887 "Parser.php"
#line 250 "Parser.y"
    function yy_r39(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -3]->minor, 'check' => $this->yystack[$this->yyidx + -5]->minor);
    }
#line 1895 "Parser.php"
#line 256 "Parser.y"
    function yy_r40(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1903 "Parser.php"
#line 263 "Parser.y"
    function yy_r41(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endifchanged") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifchanged");
    }
    $this->_retvalue = array('operation' => 'ifchanged', 'body' => $this->yystack[$this->yyidx + -7]->minor, 'check' => $this->yystack[$this->yyidx + -9]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1911 "Parser.php"
#line 271 "Parser.y"
    function yy_r42(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '==', 1 => $this->yystack[$this->yyidx + -6]->minor, 2 => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1919 "Parser.php"
#line 277 "Parser.y"
    function yy_r43(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '==', 1 => $this->yystack[$this->yyidx + -10]->minor, 2 => $this->yystack[$this->yyidx + -9]->minor, 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1927 "Parser.php"
#line 283 "Parser.y"
    function yy_r44(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifnotequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifnotequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '!=', 1 => $this->yystack[$this->yyidx + -6]->minor, 2 => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1935 "Parser.php"
#line 289 "Parser.y"
    function yy_r45(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endifnotequal") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endifnotequal");
    }
    $this->_retvalue = array('operation' => 'ifequal', 'cmp' => '!=', 1 => $this->yystack[$this->yyidx + -10]->minor, 2 => $this->yystack[$this->yyidx + -9]->minor, 'body' => $this->yystack[$this->yyidx + -7]->minor, 'else' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1943 "Parser.php"
#line 297 "Parser.y"
    function yy_r46(){ 
    if ($this->yystack[$this->yyidx + -1]->minor != "endblock") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endblock");
    }
    $this->_retvalue = array('operation' => 'block', 'name' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1951 "Parser.php"
#line 304 "Parser.y"
    function yy_r47(){
    if ($this->yystack[$this->yyidx + -2]->minor != "endblock") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -2]->minor.", expecting endblock");
    }
    $this->_retvalue = array('operation' => 'block', 'name' => $this->yystack[$this->yyidx + -6]->minor, 'body' => $this->yystack[$this->yyidx + -4]->minor); 
    }
#line 1959 "Parser.php"
#line 311 "Parser.y"
    function yy_r48(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endblock") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endblock");
    }
    $this->_retvalue = array('operation' => 'block', 'name' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor); 
    }
#line 1967 "Parser.php"
#line 326 "Parser.y"
    function yy_r50(){
    if ($this->yystack[$this->yyidx + -1]->minor != "endfilter") {
        $this->Error("Unexpected ".$this->yystack[$this->yyidx + -1]->minor.", expecting endfilter");
    }
    $this->_retvalue = array('operation' => 'filter', 'functions' => $this->yystack[$this->yyidx + -5]->minor, 'body' => $this->yystack[$this->yyidx + -3]->minor);
    }
#line 1975 "Parser.php"
#line 334 "Parser.y"
    function yy_r51(){ $this->_retvalue=array('operation' => 'regroup', 'array' => $this->yystack[$this->yyidx + -4]->minor, 'row' => $this->yystack[$this->yyidx + -2]->minor, 'as' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1978 "Parser.php"
#line 337 "Parser.y"
    function yy_r52(){ $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor; $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1981 "Parser.php"
#line 338 "Parser.y"
    function yy_r53(){ $this->_retvalue = array($this->yystack[$this->yyidx + 0]->minor);     }
#line 1984 "Parser.php"
#line 340 "Parser.y"
    function yy_r54(){ $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor, 'args'=>array($this->yystack[$this->yyidx + 0]->minor));     }
#line 1987 "Parser.php"
#line 344 "Parser.y"
    function yy_r56(){ $this->_retvalue = $this->yystack[$this->yyidx + -1]->minor; $this->_retvalue[] = $this->yystack[$this->yyidx + 0]->minor;     }
#line 1990 "Parser.php"
#line 350 "Parser.y"
    function yy_r59(){ $this->_retvalue = array('var' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1993 "Parser.php"
#line 351 "Parser.y"
    function yy_r60(){ $this->_retvalue = array('number' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 1996 "Parser.php"
#line 352 "Parser.y"
    function yy_r61(){ $this->_retvalue = trim(@$this->yystack[$this->yyidx + 0]->minor);     }
#line 1999 "Parser.php"
#line 353 "Parser.y"
    function yy_r62(){ $this->_retvalue = array('string' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 2002 "Parser.php"
#line 356 "Parser.y"
    function yy_r63(){ $this->_retvalue = array('var_filter' => $this->yystack[$this->yyidx + 0]->minor);     }
#line 2005 "Parser.php"
#line 366 "Parser.y"
    function yy_r69(){ $this->_retvalue = array('op_expr' => 'not', $this->yystack[$this->yyidx + 0]->minor);     }
#line 2008 "Parser.php"
#line 367 "Parser.y"
    function yy_r70(){ $this->_retvalue = array('op_expr' => @$this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);     }
#line 2011 "Parser.php"
#line 370 "Parser.y"
    function yy_r73(){ $this->_retvalue = array('op_expr' => trim(@$this->yystack[$this->yyidx + -1]->minor), $this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);     }
#line 2014 "Parser.php"
#line 372 "Parser.y"
    function yy_r75(){ $this->_retvalue = array('op_expr' => 'expr', array('op_expr' => @$this->yystack[$this->yyidx + -1]->minor, $this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor));     }
#line 2017 "Parser.php"
#line 373 "Parser.y"
    function yy_r76(){ $this->_retvalue = array('op_expr' => 'expr', $this->yystack[$this->yyidx + -1]->minor);     }
#line 2020 "Parser.php"
#line 378 "Parser.y"
    function yy_r78(){ $this->_retvalue = current($this->compiler->generate_variable_name($this->yystack[$this->yyidx + 0]->minor, false));     }
#line 2023 "Parser.php"
#line 380 "Parser.y"
    function yy_r79(){ 
    if (!is_array($this->yystack[$this->yyidx + -2]->minor)) { $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor); } 
    else { $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor; }  $this->_retvalue[]=array('object' => $this->yystack[$this->yyidx + 0]->minor);
    }
#line 2029 "Parser.php"
#line 385 "Parser.y"
    function yy_r80(){ 
    if (!is_array($this->yystack[$this->yyidx + -2]->minor)) { $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor); } 
    else { $this->_retvalue = $this->yystack[$this->yyidx + -2]->minor; }  $this->_retvalue[]=array('class' => '$'.$this->yystack[$this->yyidx + 0]->minor);
    }
#line 2035 "Parser.php"
#line 390 "Parser.y"
    function yy_r81(){
    if (!is_array($this->yystack[$this->yyidx + -3]->minor)) { $this->_retvalue = array($this->yystack[$this->yyidx + -3]->minor); } 
    else { $this->_retvalue = $this->yystack[$this->yyidx + -3]->minor; }  $this->_retvalue[]=$this->yystack[$this->yyidx + -1]->minor;
    }
#line 2041 "Parser.php"
#line 399 "Parser.y"
    function yy_r84(){ $this->_retvalue = array($this->yystack[$this->yyidx + -2]->minor, $this->yystack[$this->yyidx + 0]->minor);     }
#line 2044 "Parser.php"
#line 405 "Parser.y"
    function yy_r88(){ $this->_retvalue = -1 * ($this->yystack[$this->yyidx + 0]->minor);     }
#line 2047 "Parser.php"

    /**
     * placeholder for the left hand side in a reduce operation.
     * 
     * For a parser with a rule like this:
     * <pre>
     * rule(A) ::= B. { A = 1; }
     * </pre>
     * 
     * The parser will translate to something like:
     * 
     * <code>
     * function yy_r0(){$this->_retvalue = 1;}
     * </code>
     */
    private $_retvalue;

    /**
     * Perform a reduce action and the shift that must immediately
     * follow the reduce.
     * 
     * For a rule such as:
     * 
     * <pre>
     * A ::= B blah C. { dosomething(); }
     * </pre>
     * 
     * This function will first call the action, if any, ("dosomething();" in our
     * example), and then it will pop three states from the stack,
     * one for each entry on the right-hand side of the expression
     * (B, blah, and C in our example rule), and then push the result of the action
     * back on to the stack with the resulting state reduced to (as described in the .out
     * file)
     * @param int Number of the rule by which to reduce
     */
    function yy_reduce($yyruleno)
    {
        //int $yygoto;                     /* The next state */
        //int $yyact;                      /* The next action */
        //mixed $yygotominor;        /* The LHS of the rule reduced */
        //Haanga_yyStackEntry $yymsp;            /* The top of the parser's stack */
        //int $yysize;                     /* Amount to pop the stack */
        $yymsp = $this->yystack[$this->yyidx];
        if (self::$yyTraceFILE && $yyruleno >= 0 
              && $yyruleno < count(self::$yyRuleName)) {
            fprintf(self::$yyTraceFILE, "%sReduce (%d) [%s].\n",
                self::$yyTracePrompt, $yyruleno,
                self::$yyRuleName[$yyruleno]);
        }

        $this->_retvalue = $yy_lefthand_side = null;
        if (array_key_exists($yyruleno, self::$yyReduceMap)) {
            // call the action
            $this->_retvalue = null;
            $this->{'yy_r' . self::$yyReduceMap[$yyruleno]}();
            $yy_lefthand_side = $this->_retvalue;
        }
        $yygoto = self::$yyRuleInfo[$yyruleno]['lhs'];
        $yysize = self::$yyRuleInfo[$yyruleno]['rhs'];
        $this->yyidx -= $yysize;
        for ($i = $yysize; $i; $i--) {
            // pop all of the right-hand side parameters
            array_pop($this->yystack);
        }
        $yyact = $this->yy_find_reduce_action($this->yystack[$this->yyidx]->stateno, $yygoto);
        if ($yyact < self::YYNSTATE) {
            /* If we are not debugging and the reduce action popped at least
            ** one element off the stack, then we can push the new element back
            ** onto the stack here, and skip the stack overflow test in yy_shift().
            ** That gives a significant speed improvement. */
            if (!self::$yyTraceFILE && $yysize) {
                $this->yyidx++;
                $x = new Haanga_yyStackEntry;
                $x->stateno = $yyact;
                $x->major = $yygoto;
                $x->minor = $yy_lefthand_side;
                $this->yystack[$this->yyidx] = $x;
            } else {
                $this->yy_shift($yyact, $yygoto, $yy_lefthand_side);
            }
        } elseif ($yyact == self::YYNSTATE + self::YYNRULE + 1) {
            $this->yy_accept();
        }
    }

    /**
     * The following code executes when the parse fails
     * 
     * Code from %parse_fail is inserted here
     */
    function yy_parse_failed()
    {
        if (self::$yyTraceFILE) {
            fprintf(self::$yyTraceFILE, "%sFail!\n", self::$yyTracePrompt);
        }
        while ($this->yyidx >= 0) {
            $this->yy_pop_parser_stack();
        }
        /* Here code is inserted which will be executed whenever the
        ** parser fails */
    }

    /**
     * The following code executes when a syntax error first occurs.
     * 
     * %syntax_error code is inserted here
     * @param int The major type of the error token
     * @param mixed The minor type of the error token
     */
    function yy_syntax_error($yymajor, $TOKEN)
    {
#line 71 "Parser.y"

    $expect = array();
    foreach ($this->yy_get_expected_tokens($yymajor) as $token) {
        $expect[] = self::$yyTokenName[$token];
    }
    $this->Error('Unexpected ' . $this->tokenName($yymajor) . '(' . $TOKEN. ')');
#line 2167 "Parser.php"
    }

    /**
     * The following is executed when the parser accepts
     * 
     * %parse_accept code is inserted here
     */
    function yy_accept()
    {
        if (self::$yyTraceFILE) {
            fprintf(self::$yyTraceFILE, "%sAccept!\n", self::$yyTracePrompt);
        }
        while ($this->yyidx >= 0) {
            $stack = $this->yy_pop_parser_stack();
        }
        /* Here code is inserted which will be executed whenever the
        ** parser accepts */
#line 57 "Parser.y"

#line 2188 "Parser.php"
    }

    /**
     * The main parser program.
     * 
     * The first argument is the major token number.  The second is
     * the token value string as scanned from the input.
     *
     * @param int   $yymajor      the token number
     * @param mixed $yytokenvalue the token value
     * @param mixed ...           any extra arguments that should be passed to handlers
     *
     * @return void
     */
    function doParse($yymajor, $yytokenvalue)
    {
//        $yyact;            /* The parser action. */
//        $yyendofinput;     /* True if we are at the end of input */
        $yyerrorhit = 0;   /* True if yymajor has invoked an error */
        
        /* (re)initialize the parser, if necessary */
        if ($this->yyidx === null || $this->yyidx < 0) {
            /* if ($yymajor == 0) return; // not sure why this was here... */
            $this->yyidx = 0;
            $this->yyerrcnt = -1;
            $x = new Haanga_yyStackEntry;
            $x->stateno = 0;
            $x->major = 0;
            $this->yystack = array();
            array_push($this->yystack, $x);
        }
        $yyendofinput = ($yymajor==0);
        
        if (self::$yyTraceFILE) {
            fprintf(
                self::$yyTraceFILE,
                "%sInput %s\n",
                self::$yyTracePrompt,
                self::$yyTokenName[$yymajor]
            );
        }
        
        do {
            $yyact = $this->yy_find_shift_action($yymajor);
            if ($yymajor < self::YYERRORSYMBOL
                && !$this->yy_is_expected_token($yymajor)
            ) {
                // force a syntax error
                $yyact = self::YY_ERROR_ACTION;
            }
            if ($yyact < self::YYNSTATE) {
                $this->yy_shift($yyact, $yymajor, $yytokenvalue);
                $this->yyerrcnt--;
                if ($yyendofinput && $this->yyidx >= 0) {
                    $yymajor = 0;
                } else {
                    $yymajor = self::YYNOCODE;
                }
            } elseif ($yyact < self::YYNSTATE + self::YYNRULE) {
                $this->yy_reduce($yyact - self::YYNSTATE);
            } elseif ($yyact == self::YY_ERROR_ACTION) {
                if (self::$yyTraceFILE) {
                    fprintf(
                        self::$yyTraceFILE,
                        "%sSyntax Error!\n",
                        self::$yyTracePrompt
                    );
                }
                if (self::YYERRORSYMBOL) {
                    /* A syntax error has occurred.
                    ** The response to an error depends upon whether or not the
                    ** grammar defines an error token "ERROR".  
                    **
                    ** This is what we do if the grammar does define ERROR:
                    **
                    **  * Call the %syntax_error function.
                    **
                    **  * Begin popping the stack until we enter a state where
                    **    it is legal to shift the error symbol, then shift
                    **    the error symbol.
                    **
                    **  * Set the error count to three.
                    **
                    **  * Begin accepting and shifting new tokens.  No new error
                    **    processing will occur until three tokens have been
                    **    shifted successfully.
                    **
                    */
                    if ($this->yyerrcnt < 0) {
                        $this->yy_syntax_error($yymajor, $yytokenvalue);
                    }
                    $yymx = $this->yystack[$this->yyidx]->major;
                    if ($yymx == self::YYERRORSYMBOL || $yyerrorhit ) {
                        if (self::$yyTraceFILE) {
                            fprintf(
                                self::$yyTraceFILE,
                                "%sDiscard input token %s\n",
                                self::$yyTracePrompt,
                                self::$yyTokenName[$yymajor]
                            );
                        }
                        $this->yy_destructor($yymajor, $yytokenvalue);
                        $yymajor = self::YYNOCODE;
                    } else {
                        while ($this->yyidx >= 0
                            && $yymx != self::YYERRORSYMBOL
                            && ($yyact = $this->yy_find_shift_action(self::YYERRORSYMBOL)) >= self::YYNSTATE
                        ) {
                            $this->yy_pop_parser_stack();
                        }
                        if ($this->yyidx < 0 || $yymajor==0) {
                            $this->yy_destructor($yymajor, $yytokenvalue);
                            $this->yy_parse_failed();
                            $yymajor = self::YYNOCODE;
                        } elseif ($yymx != self::YYERRORSYMBOL) {
                            $u2 = 0;
                            $this->yy_shift($yyact, self::YYERRORSYMBOL, $u2);
                        }
                    }
                    $this->yyerrcnt = 3;
                    $yyerrorhit = 1;
                } else {
                    /* YYERRORSYMBOL is not defined */
                    /* This is what we do if the grammar does not define ERROR:
                    **
                    **  * Report an error message, and throw away the input token.
                    **
                    **  * If the input token is $, then fail the parse.
                    **
                    ** As before, subsequent error messages are suppressed until
                    ** three input tokens have been successfully shifted.
                    */
                    if ($this->yyerrcnt <= 0) {
                        $this->yy_syntax_error($yymajor, $yytokenvalue);
                    }
                    $this->yyerrcnt = 3;
                    $this->yy_destructor($yymajor, $yytokenvalue);
                    if ($yyendofinput) {
                        $this->yy_parse_failed();
                    }
                    $yymajor = self::YYNOCODE;
                }
            } else {
                $this->yy_accept();
                $yymajor = self::YYNOCODE;
            }            
        } while ($yymajor != self::YYNOCODE && $this->yyidx >= 0);
    }
}
